from pydantic import Field

from autotrain.trainers.common import AutoTrainParams


class DreamBoothTrainingParams(AutoTrainParams):
    model: str = Field(None, title="Model name")
    revision: str = Field(None, title="Revision")
    tokenizer: str = Field(None, title="Tokenizer, if different from model")
    image_path: str = Field(None, title="Image path")
    class_image_path: str = Field(None, title="Class image path")
    prompt: str = Field(None, title="Instance prompt")
    class_prompt: str = Field(None, title="Class prompt")
    num_class_images: int = Field(100, title="Number of class images")
    class_labels_conditioning: str = Field(None, title="Class labels conditioning")

    prior_preservation: bool = Field(False, title="With prior preservation")
    prior_loss_weight: float = Field(1.0, title="Prior loss weight")

    project_name: str = Field("dreambooth-model", title="Output directory")
    seed: int = Field(42, title="Seed")
    resolution: int = Field(512, title="Resolution")
    center_crop: bool = Field(False, title="Center crop")
    train_text_encoder: bool = Field(False, title="Train text encoder")
    batch_size: int = Field(4, title="Train batch size")
    sample_batch_size: int = Field(4, title="Sample batch size")
    epochs: int = Field(1, title="Number of training epochs")
    num_steps: int = Field(None, title="Max train steps")
    checkpointing_steps: int = Field(500, title="Checkpointing steps")
    resume_from_checkpoint: str = Field(None, title="Resume from checkpoint")

    gradient_accumulation: int = Field(1, title="Gradient accumulation steps")
    gradient_checkpointing: bool = Field(False, title="Gradient checkpointing")

    lr: float = Field(5e-4, title="Learning rate")
    scale_lr: bool = Field(False, title="Scale learning rate")
    scheduler: str = Field("constant", title="Learning rate scheduler")
    warmup_steps: int = Field(0, title="Learning rate warmup steps")
    num_cycles: int = Field(1, title="Learning rate num cycles")
    lr_power: float = Field(1.0, title="Learning rate power")

    dataloader_num_workers: int = Field(0, title="Dataloader num workers")
    use_8bit_adam: bool = Field(False, title="Use 8bit adam")
    adam_beta1: float = Field(0.9, title="Adam beta 1")
    adam_beta2: float = Field(0.999, title="Adam beta 2")
    adam_weight_decay: float = Field(1e-2, title="Adam weight decay")
    adam_epsilon: float = Field(1e-8, title="Adam epsilon")
    max_grad_norm: float = Field(1.0, title="Max grad norm")

    allow_tf32: bool = Field(False, title="Allow TF32")
    prior_generation_precision: str = Field(None, title="Prior generation precision")
    local_rank: int = Field(-1, title="Local rank")
    xformers: bool = Field(False, title="Enable xformers memory efficient attention")
    pre_compute_text_embeddings: bool = Field(False, title="Pre compute text embeddings")
    tokenizer_max_length: int = Field(None, title="Tokenizer max length")
    text_encoder_use_attention_mask: bool = Field(False, title="Text encoder use attention mask")

    rank: int = Field(4, title="Rank")
    xl: bool = Field(False, title="XL")

    fp16: bool = Field(False, title="FP16")
    bf16: bool = Field(False, title="BF16")

    token: str = Field(None, title="Hub token")
    repo_id: str = Field(None, title="Hub model id")
    push_to_hub: bool = Field(False, title="Push to hub")
    username: str = Field(None, title="Hub username")

    # disabled:
    validation_prompt: str = Field(None, title="Validation prompt")
    num_validation_images: int = Field(4, title="Number of validation images")
    validation_epochs: int = Field(50, title="Validation epochs")
    checkpoints_total_limit: int = Field(None, title="Checkpoints total limit")
    validation_images: str = Field(None, title="Validation images")

    logging: bool = Field(False, title="Logging using tensorboard")



from functools import partial

import gradio as gr
import pandas as pd

from autotrain.apps import common
from autotrain.apps import utils as app_utils
from autotrain.dataset import AutoTrainDreamboothDataset
from autotrain.project import AutoTrainProject


ALLOWED_FILE_TYPES = ["png", "jpg", "jpeg"]

MODELS = [
    "stabilityai/stable-diffusion-xl-base-1.0",
    "runwayml/stable-diffusion-v1-5",
    "stabilityai/stable-diffusion-2-1",
    "stabilityai/stable-diffusion-2-1-base",
]


def start_training(
    jobs_df,
    model_choice,
    training_data,
    project_name,
    autotrain_username,
    user_token,
    prompt,
):
    if len(jobs_df) == 0:
        raise gr.Error("Please add at least one job.")
    dset = AutoTrainDreamboothDataset(
        concept_images=training_data,
        concept_name=prompt,
        token=user_token,
        project_name=project_name,
        username=autotrain_username,
    )
    dset.prepare()
    project = AutoTrainProject(dataset=dset, job_params=jobs_df)
    ids = project.create()
    return gr.Markdown.update(
        value=f"Training started for {len(ids)} jobs. You can view the status of your jobs at ids: {', '.join(ids)}",
        visible=True,
    )


def main():
    with gr.Blocks(theme=app_utils.THEME) as demo:
        gr.Markdown("### 🚀 DreamBooth")
        user_token, valid_can_pay, who_is_training = common.user_validation()
        autotrain_username, project_name, model_choice, autotrain_backend = common.base_components(who_is_training)
        with gr.Row():
            training_data = gr.File(
                label="Images",
                file_types=ALLOWED_FILE_TYPES,
                file_count="multiple",
                visible=True,
                interactive=True,
            )
            with gr.Column():
                with gr.Group():
                    with gr.Row():
                        hyp_prompt = gr.Textbox(
                            label="Prompt",
                            placeholder="photo of sks dog",
                            lines=1,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_prompt",
                        )
                    with gr.Row():
                        hyp_resolution = gr.Number(
                            label="Resolution",
                            value=1024,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_resolution",
                            precision=0,
                        )
                        hyp_scheduler = gr.Dropdown(
                            label="Scheduler",
                            choices=["cosine", "linear", "constant"],
                            value="linear",
                            visible=True,
                            interactive=True,
                            elem_id="hyp_scheduler",
                        )
                    with gr.Row():
                        with gr.Group():
                            hyp_gradient_checkpointing = gr.Checkbox(
                                label="Gradient Checkpointing",
                                value=False,
                                visible=True,
                                interactive=True,
                                elem_id="hyp_gradient_checkpointing",
                            )
                            hyp_xformers = gr.Checkbox(
                                label="Enable XFormers",
                                value=True,
                                visible=True,
                                interactive=True,
                                elem_id="hyp_xformers",
                            )

                            with gr.Row():
                                hyp_prior_preservation = gr.Checkbox(
                                    label="Prior Preservation",
                                    value=False,
                                    visible=True,
                                    interactive=True,
                                    elem_id="hyp_prior_preservation",
                                )
                                hyp_scale_lr = gr.Checkbox(
                                    label="Scale LR",
                                    value=False,
                                    visible=True,
                                    interactive=True,
                                    elem_id="hyp_scale_lr",
                                )
                            with gr.Row():
                                hyp_use_8bit_adam = gr.Checkbox(
                                    label="Use 8bit Adam",
                                    value=True,
                                    visible=True,
                                    interactive=True,
                                    elem_id="hyp_use_8bit_adam",
                                )
                                hyp_train_text_encoder = gr.Checkbox(
                                    label="Train Text Encoder",
                                    value=False,
                                    visible=True,
                                    interactive=True,
                                    elem_id="hyp_train_text_encoder",
                                )
                            with gr.Row():
                                hyp_fp16 = gr.Checkbox(
                                    label="FP16",
                                    value=True,
                                    visible=True,
                                    interactive=True,
                                    elem_id="hyp_fp16",
                                )
                                hyp_center_crop = gr.Checkbox(
                                    label="Center Crop",
                                    value=False,
                                    visible=True,
                                    interactive=True,
                                    elem_id="hyp_center_crop",
                                )

            with gr.Column():
                with gr.Group():
                    param_choice = gr.Dropdown(
                        label="Parameter Choice",
                        choices=["Manual"],
                        value="Manual",
                        visible=True,
                        interactive=True,
                        elem_id="param_choice",
                    )
                    with gr.Row():
                        hyp_lr = gr.Number(
                            label="Learning Rate",
                            value=1e-4,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_lr",
                        )
                        hyp_num_steps = gr.Number(
                            label="Number of Steps",
                            value=500,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_num_steps",
                            precision=0,
                        )
                    with gr.Row():
                        hyp_warmup_steps = gr.Number(
                            label="Warmup Steps",
                            value=0,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_warmup_steps",
                            precision=0,
                        )
                        hyp_batch_size = gr.Number(
                            label="Batch Size",
                            value=1,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_batch_size",
                            precision=0,
                        )
                    with gr.Row():
                        hyp_prior_loss_weight = gr.Number(
                            label="Prior Loss Weight",
                            value=1.0,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_prior_loss_weight",
                        )
                        hyp_weight_decay = gr.Number(
                            label="Weight Decay",
                            value=0.01,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_weight_decay",
                        )
                    with gr.Row():
                        hyp_gradient_accumulation = gr.Number(
                            label="Gradient Accumulation Steps",
                            value=4,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_gradient_accumulation",
                            precision=0,
                        )

        with gr.Row():
            add_job_button = gr.Button(value="Add Job", elem_id="add_job_button")
            clear_jobs_button = gr.Button(value="Clear Jobs", elem_id="clear_jobs_button")
            start_training_button = gr.Button(value="Start Training", elem_id="start_training_button")

        output_md = gr.Markdown(
            value="WARNING: Clicking `Start Training` will incur costs!", visible=True, interactive=False
        )
        jobs_df = gr.DataFrame(visible=False, interactive=False, value=pd.DataFrame())

        hyperparameters = [
            hyp_prompt,
            hyp_resolution,
            hyp_scheduler,
            hyp_gradient_checkpointing,
            hyp_xformers,
            hyp_prior_preservation,
            hyp_scale_lr,
            hyp_use_8bit_adam,
            hyp_train_text_encoder,
            hyp_fp16,
            hyp_center_crop,
            hyp_lr,
            hyp_num_steps,
            hyp_warmup_steps,
            hyp_batch_size,
            hyp_prior_loss_weight,
            hyp_weight_decay,
            hyp_gradient_accumulation,
        ]

        model_choice.change(
            app_utils.handle_model_choice_change,
            inputs=model_choice,
            outputs=[jobs_df, param_choice, autotrain_backend],
        )

        def _add_job(components):
            if not components[training_data]:
                raise gr.Error("Please upload training images first.")
            if len(str(components[hyp_prompt].strip())) == 0:
                raise gr.Error("Prompt cannot be empty.")
            if components[param_choice] == "AutoTrain" and components[autotrain_backend] != "AutoTrain":
                raise gr.Error("AutoTrain param choice is only available with AutoTrain backend.")

            _training_params = {}
            for _hyperparameter in hyperparameters:
                _training_params[_hyperparameter.elem_id] = components[_hyperparameter]

            _training_params_df = app_utils.fetch_training_params_df(
                components[param_choice],
                components[jobs_df],
                _training_params,
                components[model_choice],
                components[autotrain_backend],
            )
            return gr.DataFrame.update(value=_training_params_df, visible=True, interactive=False)

        add_job_button.click(
            _add_job,
            inputs=set(
                [
                    training_data,
                    param_choice,
                    autotrain_backend,
                    jobs_df,
                    model_choice,
                    autotrain_backend,
                ]
                + hyperparameters
            ),
            outputs=jobs_df,
        )

        clear_jobs_button.click(
            app_utils.clear_jobs,
            inputs=jobs_df,
            outputs=jobs_df,
        )

        start_training_button.click(
            start_training,
            inputs=[
                jobs_df,
                model_choice,
                training_data,
                project_name,
                autotrain_username,
                user_token,
                hyp_prompt,
            ],
            outputs=output_md,
        )

        demo.load(
            app_utils._update_project_name,
            outputs=project_name,
        ).then(
            partial(app_utils._update_hub_model_choices, task="dreambooth"),
            outputs=model_choice,
        )

    return demo


if __name__ == "__main__":
    demo = main()
    demo.launch()



import os
import uuid
import zipfile
from dataclasses import dataclass
from typing import Any, Dict, List, Optional

import pandas as pd

from autotrain import logger
from autotrain.preprocessor.dreambooth import DreamboothPreprocessor
from autotrain.preprocessor.tabular import (
    TabularBinaryClassificationPreprocessor,
    TabularMultiClassClassificationPreprocessor,
    TabularMultiColumnRegressionPreprocessor,
    TabularMultiLabelClassificationPreprocessor,
    TabularSingleColumnRegressionPreprocessor,
)
from autotrain.preprocessor.text import (
    LLMPreprocessor,
    TextBinaryClassificationPreprocessor,
    TextMultiClassClassificationPreprocessor,
    TextSingleColumnRegressionPreprocessor,
)
from autotrain.preprocessor.vision import ImageClassificationPreprocessor


def remove_non_image_files(folder):
    # Define allowed image file extensions
    allowed_extensions = {".jpg", ".jpeg", ".png", ".JPG", ".JPEG", ".PNG"}

    # Iterate through all files in the folder
    for root, dirs, files in os.walk(folder):
        for file in files:
            # Get the file extension
            file_extension = os.path.splitext(file)[1]

            # If the file extension is not in the allowed list, remove the file
            if file_extension.lower() not in allowed_extensions:
                file_path = os.path.join(root, file)
                os.remove(file_path)
                print(f"Removed file: {file_path}")

        # Recursively call the function on each subfolder
        for subfolder in dirs:
            remove_non_image_files(os.path.join(root, subfolder))


@dataclass
class AutoTrainDreamboothDataset:
    concept_images: List[Any]
    concept_name: str
    token: str
    project_name: str
    username: str

    def __str__(self) -> str:
        info = f"Dataset: {self.project_name} ({self.task})\n"
        return info

    def __post_init__(self):
        self.task = "dreambooth"
        logger.info(self.__str__())

    @property
    def num_samples(self):
        return len(self.concept_images)

    def prepare(self):
        preprocessor = DreamboothPreprocessor(
            concept_images=self.concept_images,
            concept_name=self.concept_name,
            token=self.token,
            project_name=self.project_name,
            username=self.username,
        )
        preprocessor.prepare()


@dataclass
class AutoTrainImageClassificationDataset:
    train_data: str
    token: str
    project_name: str
    username: str
    valid_data: Optional[str] = None
    percent_valid: Optional[float] = None

    def __str__(self) -> str:
        info = f"Dataset: {self.project_name} ({self.task})\n"
        info += f"Train data: {self.train_data}\n"
        info += f"Valid data: {self.valid_data}\n"
        return info

    def __post_init__(self):
        self.task = "image_multi_class_classification"
        if not self.valid_data and self.percent_valid is None:
            self.percent_valid = 0.2
        elif self.valid_data and self.percent_valid is not None:
            raise ValueError("You can only specify one of valid_data or percent_valid")
        elif self.valid_data:
            self.percent_valid = 0.0
        logger.info(self.__str__())

        self.num_files = self._count_files()

    @property
    def num_samples(self):
        return self.num_files

    def _count_files(self):
        num_files = 0
        zip_ref = zipfile.ZipFile(self.train_data, "r")
        for _ in zip_ref.namelist():
            num_files += 1
        if self.valid_data:
            zip_ref = zipfile.ZipFile(self.valid_data, "r")
            for _ in zip_ref.namelist():
                num_files += 1
        return num_files

    def prepare(self):
        cache_dir = os.environ.get("HF_HOME")
        if not cache_dir:
            cache_dir = os.path.join(os.path.expanduser("~"), ".cache", "huggingface")

        random_uuid = uuid.uuid4()
        train_dir = os.path.join(cache_dir, "autotrain", str(random_uuid))
        os.makedirs(train_dir, exist_ok=True)
        zip_ref = zipfile.ZipFile(self.train_data, "r")
        zip_ref.extractall(train_dir)
        # remove the __MACOSX directory
        macosx_dir = os.path.join(train_dir, "__MACOSX")
        if os.path.exists(macosx_dir):
            os.system(f"rm -rf {macosx_dir}")
        remove_non_image_files(train_dir)

        valid_dir = None
        if self.valid_data:
            random_uuid = uuid.uuid4()
            valid_dir = os.path.join(cache_dir, "autotrain", str(random_uuid))
            os.makedirs(valid_dir, exist_ok=True)
            zip_ref = zipfile.ZipFile(self.valid_data, "r")
            zip_ref.extractall(valid_dir)
            # remove the __MACOSX directory
            macosx_dir = os.path.join(valid_dir, "__MACOSX")
            if os.path.exists(macosx_dir):
                os.system(f"rm -rf {macosx_dir}")
            remove_non_image_files(valid_dir)

        preprocessor = ImageClassificationPreprocessor(
            train_data=train_dir,
            valid_data=valid_dir,
            token=self.token,
            project_name=self.project_name,
            username=self.username,
        )
        preprocessor.prepare()


@dataclass
class AutoTrainDataset:
    train_data: List[str]
    task: str
    token: str
    project_name: str
    username: str
    column_mapping: Optional[Dict[str, str]] = None
    valid_data: Optional[List[str]] = None
    percent_valid: Optional[float] = None
    convert_to_class_label: Optional[bool] = False

    def __str__(self) -> str:
        info = f"Dataset: {self.project_name} ({self.task})\n"
        info += f"Train data: {self.train_data}\n"
        info += f"Valid data: {self.valid_data}\n"
        info += f"Column mapping: {self.column_mapping}\n"
        return info

    def __post_init__(self):
        if not self.valid_data and self.percent_valid is None:
            self.percent_valid = 0.2
        elif self.valid_data and self.percent_valid is not None:
            raise ValueError("You can only specify one of valid_data or percent_valid")
        elif self.valid_data:
            self.percent_valid = 0.0

        self.train_df, self.valid_df = self._preprocess_data()
        logger.info(self.__str__())

    def _preprocess_data(self):
        train_df = []
        for file in self.train_data:
            if isinstance(file, pd.DataFrame):
                train_df.append(file)
            else:
                train_df.append(pd.read_csv(file))
        if len(train_df) > 1:
            train_df = pd.concat(train_df)
        else:
            train_df = train_df[0]

        valid_df = None
        if len(self.valid_data) > 0:
            valid_df = []
            for file in self.valid_data:
                if isinstance(file, pd.DataFrame):
                    valid_df.append(file)
                else:
                    valid_df.append(pd.read_csv(file))
            if len(valid_df) > 1:
                valid_df = pd.concat(valid_df)
            else:
                valid_df = valid_df[0]
        return train_df, valid_df

    @property
    def num_samples(self):
        return len(self.train_df) + len(self.valid_df) if self.valid_df is not None else len(self.train_df)

    def prepare(self):
        if self.task == "text_binary_classification":
            text_column = self.column_mapping["text"]
            label_column = self.column_mapping["label"]
            preprocessor = TextBinaryClassificationPreprocessor(
                train_data=self.train_df,
                text_column=text_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=self.valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
                convert_to_class_label=self.convert_to_class_label,
            )
            preprocessor.prepare()

        elif self.task == "text_multi_class_classification":
            text_column = self.column_mapping["text"]
            label_column = self.column_mapping["label"]
            preprocessor = TextMultiClassClassificationPreprocessor(
                train_data=self.train_df,
                text_column=text_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=self.valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
                convert_to_class_label=self.convert_to_class_label,
            )
            preprocessor.prepare()

        elif self.task == "text_single_column_regression":
            text_column = self.column_mapping["text"]
            label_column = self.column_mapping["label"]
            preprocessor = TextSingleColumnRegressionPreprocessor(
                train_data=self.train_df,
                text_column=text_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=self.valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()

        elif self.task == "lm_training":
            text_column = self.column_mapping.get("text", None)
            if text_column is None:
                prompt_column = self.column_mapping["prompt"]
                response_column = self.column_mapping["response"]
            else:
                prompt_column = None
                response_column = None
            context_column = self.column_mapping.get("context", None)
            prompt_start_column = self.column_mapping.get("prompt_start", None)
            preprocessor = LLMPreprocessor(
                train_data=self.train_df,
                text_column=text_column,
                prompt_column=prompt_column,
                response_column=response_column,
                context_column=context_column,
                prompt_start_column=prompt_start_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=self.valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()

        elif self.task == "tabular_binary_classification":
            id_column = self.column_mapping["id"]
            label_column = self.column_mapping["label"][0]
            if len(id_column.strip()) == 0:
                id_column = None
            preprocessor = TabularBinaryClassificationPreprocessor(
                train_data=self.train_df,
                id_column=id_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=self.valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()
        elif self.task == "tabular_multi_class_classification":
            id_column = self.column_mapping["id"]
            label_column = self.column_mapping["label"][0]
            if len(id_column.strip()) == 0:
                id_column = None
            preprocessor = TabularMultiClassClassificationPreprocessor(
                train_data=self.train_df,
                id_column=id_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=self.valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()
        elif self.task == "tabular_single_column_regression":
            id_column = self.column_mapping["id"]
            label_column = self.column_mapping["label"][0]
            if len(id_column.strip()) == 0:
                id_column = None
            preprocessor = TabularSingleColumnRegressionPreprocessor(
                train_data=self.train_df,
                id_column=id_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=self.valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()
        elif self.task == "tabular_multi_column_regression":
            id_column = self.column_mapping["id"]
            label_column = self.column_mapping["label"]
            if len(id_column.strip()) == 0:
                id_column = None
            preprocessor = TabularMultiColumnRegressionPreprocessor(
                train_data=self.train_df,
                id_column=id_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=self.valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()
        elif self.task == "tabular_multi_label_classification":
            id_column = self.column_mapping["id"]
            label_column = self.column_mapping["label"]
            if len(id_column.strip()) == 0:
                id_column = None
            preprocessor = TabularMultiLabelClassificationPreprocessor(
                train_data=self.train_df,
                id_column=id_column,
                label_column=label_column,
                username=self.username,
                project_name=self.project_name,
                valid_data=self.valid_df,
                test_size=self.percent_valid,
                token=self.token,
                seed=42,
            )
            preprocessor.prepare()
        else:
            raise ValueError(f"Task {self.task} not supported")



import os

from pydantic import BaseModel, Field

from autotrain import logger


class LLMTrainingParams(BaseModel):
    model: str = Field("gpt2", title="Model name")
    data_path: str = Field("data", title="Data path")
    project_name: str = Field("Project Name", title="Output directory")
    train_split: str = Field("train", title="Train data config")
    valid_split: str = Field(None, title="Validation data config")
    text_column: str = Field("text", title="Text column")
    rejected_text_column: str = Field(None, title="Rejected text column")
    token: str = Field(None, title="Huggingface token")
    lr: float = Field(3e-5, title="Learning rate")
    epochs: int = Field(1, title="Number of training epochs")
    batch_size: int = Field(2, title="Training batch size")
    warmup_ratio: float = Field(0.1, title="Warmup proportion")
    gradient_accumulation: int = Field(1, title="Gradient accumulation steps")
    optimizer: str = Field("adamw_torch", title="Optimizer")
    scheduler: str = Field("linear", title="Scheduler")
    weight_decay: float = Field(0.0, title="Weight decay")
    max_grad_norm: float = Field(1.0, title="Max gradient norm")
    seed: int = Field(42, title="Seed")
    add_eos_token: bool = Field(True, title="Add EOS token")
    block_size: int = Field(-1, title="Block size")
    use_peft: bool = Field(False, title="Use PEFT")
    lora_r: int = Field(16, title="Lora r")
    lora_alpha: int = Field(32, title="Lora alpha")
    lora_dropout: float = Field(0.05, title="Lora dropout")
    logging_steps: int = Field(-1, title="Logging steps")
    evaluation_strategy: str = Field("epoch", title="Evaluation strategy")
    save_total_limit: int = Field(1, title="Save total limit")
    save_strategy: str = Field("epoch", title="Save strategy")
    auto_find_batch_size: bool = Field(False, title="Auto find batch size")
    fp16: bool = Field(False, title="FP16")
    push_to_hub: bool = Field(False, title="Push to hub")
    use_int8: bool = Field(False, title="Use int8")
    model_max_length: int = Field(2048, title="Model max length")
    repo_id: str = Field(None, title="Repo id")
    use_int4: bool = Field(False, title="Use int4")
    trainer: str = Field("default", title="Trainer type")
    target_modules: str = Field(None, title="Target modules")
    merge_adapter: bool = Field(False, title="Merge adapter")
    username: str = Field(None, title="Hugging Face Username")
    use_flash_attention_2: bool = Field(False, title="Use flash attention 2")
    log: str = Field("none", title="Logging using experiment tracking")
    disable_gradient_checkpointing: bool = Field(False, title="Gradient checkpointing")

    def save(self, output_dir):
        os.makedirs(output_dir, exist_ok=True)
        path = os.path.join(output_dir, "training_params.json")
        # save formatted json
        with open(path, "w") as f:
            f.write(self.json(indent=4))

    def __str__(self):
        data = self.dict()
        data["token"] = "*****" if data.get("token") else None
        return str(data)

    def __init__(self, **data):
        super().__init__(**data)

        # Parameters not supplied by the user
        defaults = {f.name for f in self.__fields__.values() if f.default == self.__dict__[f.name]}
        supplied = set(data.keys())
        not_supplied = defaults - supplied
        if not_supplied:
            logger.warning(f"Parameters not supplied by user and set to default: {', '.join(not_supplied)}")

        # Parameters that were supplied but not used
        # This is a naive implementation. It might catch some internal Pydantic params.
        unused = supplied - set(self.__fields__)
        if unused:
            logger.warning(f"Parameters supplied but not used: {', '.join(unused)}")



import gradio as gr

from autotrain.apps import utils as app_utils
from autotrain.apps.dreambooth import main as dreambooth
from autotrain.apps.llm import main as llm
from autotrain.apps.tabular import main as tabular
from autotrain.apps.text_classification import main as text_classification


llm = llm()
text_classification = text_classification()
tabular = tabular()
dreambooth = dreambooth()


def main():
    with gr.Blocks(theme=app_utils.THEME) as demo:
        gr.Markdown("## 🤗 AutoTrain Advanced")
        gr.Markdown(
            "AutoTrain Advanced is a no-code solution that allows you to train machine learning models in just a few clicks."
        )
        with gr.Accordion(label="Issues/Feature Requests", open=False):
            gr.Markdown(
                "If you have any issues or feature requests, please submit them to the [AutoTrain GitHub](https://github.com/huggingface/autotrain-advanced)."
            )
        with gr.Accordion(label="Pricing", open=False):
            gr.Markdown("You will be charged per minute of training time. The prices are as follows:")
            gr.Markdown("- A10g Large: $0.0525/minute")
            gr.Markdown("- A10g Small: $0.0175/minute")
            gr.Markdown("- A100 Large: $0.06883/minute")
            gr.Markdown("- T4 Medium: $0.015/minute")
            gr.Markdown("- T4 Small: $0.1/minute")
            gr.Markdown("- CPU: $0.0005/minute")
        with gr.Tabs():
            with gr.Tab(label="LLM"):
                llm.render()
            with gr.Tab(label="Text Classification"):
                text_classification.render()
            with gr.Tab(label="Tabular"):
                tabular.render()
            with gr.Tab(label="DreamBooth"):
                dreambooth.render()
    return demo



from functools import partial

import gradio as gr
import pandas as pd

from autotrain.apps import common
from autotrain.apps import utils as app_utils
from autotrain.dataset import AutoTrainDataset
from autotrain.project import AutoTrainProject


ALLOWED_MODELS = [
    "xgboost",
    "random_forest",
    "ridge",
    "logistic_regression",
    "svm",
    "extra_trees",
    "gradient_boosting",
    "adaboost",
    "decision_tree",
    "knn",
]


def start_training(
    jobs_df,
    model_choice,
    training_data,
    validation_data,
    project_name,
    autotrain_username,
    user_token,
    col_map_id,
    col_map_label,
    task,
):
    if len(jobs_df) == 0:
        raise gr.Error("Please add at least one job.")
    if isinstance(col_map_label, str):
        col_map_label = [col_map_label]
    if task == "classification" and len(col_map_label) > 1:
        task = "tabular_multi_label_classification"
    elif task == "classification" and len(col_map_label) == 1:
        task = "tabular_multi_class_classification"
    elif task == "regression" and len(col_map_label) > 1:
        task = "tabular_multi_column_regression"
    elif task == "regression" and len(col_map_label) == 1:
        task = "tabular_single_column_regression"
    else:
        raise gr.Error("Please select a valid task.")

    training_data = [f.name for f in training_data]
    if validation_data is None:
        validation_data = []
    else:
        validation_data = [f.name for f in validation_data]
    dset = AutoTrainDataset(
        train_data=training_data,
        task=task,
        token=user_token,
        project_name=project_name,
        username=autotrain_username,
        column_mapping={"id": col_map_id, "label": col_map_label},
        valid_data=validation_data,
        percent_valid=None,  # TODO: add to UI
        convert_to_class_label=True,
    )
    dset.prepare()
    project = AutoTrainProject(dataset=dset, job_params=jobs_df)
    ids = project.create()
    return gr.Markdown.update(
        value=f"Training started for {len(ids)} jobs. You can view the status of your jobs at ids: {', '.join(ids)}",
        visible=True,
    )


def main():
    with gr.Blocks(theme=app_utils.THEME) as demo:
        gr.Markdown("### 🚀 Tabular Classification / Regression")
        user_token, valid_can_pay, who_is_training = common.user_validation()
        autotrain_username, project_name, model_choice, autotrain_backend = common.base_components(who_is_training)
        model_choice.update(label="", visible=False, interactive=False)
        with gr.Row():
            training_data, validation_data = common.train_valid_components()
            with gr.Column():
                with gr.Group():
                    col_map_id = gr.Dropdown(
                        label="`id` column",
                        choices=[],
                        visible=True,
                        interactive=True,
                        elem_id="col_map_id",
                    )
                    col_map_target = gr.Dropdown(
                        label="`target` column(s)",
                        choices=[],
                        visible=True,
                        interactive=True,
                        elem_id="col_map_target",
                        multiselect=True,
                    )
                    with gr.Row():
                        hyp_task = gr.Dropdown(
                            label="Task",
                            choices=["classification", "regression"],
                            value="classification",
                            visible=True,
                            interactive=True,
                            elem_id="hyp_task",
                        )

            with gr.Column():
                with gr.Group():
                    with gr.Row():
                        param_choice = gr.Dropdown(
                            label="Param Choice",
                            choices=["Manual"],
                            value="Manual",
                            visible=True,
                            interactive=True,
                            elem_id="param_choice",
                        )
                    hyp_model = gr.Dropdown(
                        label="Model",
                        choices=ALLOWED_MODELS,
                        value=ALLOWED_MODELS[0],
                        visible=True,
                        interactive=True,
                        elem_id="hyp_model",
                    )
                    hyp_categorial_imputer = gr.Dropdown(
                        label="Categorical Imputer",
                        choices=["most_frequent", "none"],
                        value="none",
                        visible=True,
                        interactive=True,
                        elem_id="hyp_categorical_imputer",
                    )
                    with gr.Row():
                        hyp_numerical_imputer = gr.Dropdown(
                            label="Numerical Imputer",
                            choices=["mean", "median", "most_frequent", "none"],
                            value="mean",
                            visible=True,
                            interactive=True,
                            elem_id="hyp_numerical_imputer",
                        )
                        hyp_numeric_scaler = gr.Dropdown(
                            label="Numeric Scaler",
                            choices=["standard", "minmax", "normal", "robust", "none"],
                            value="standard",
                            visible=True,
                            interactive=True,
                            elem_id="hyp_numeric_scaler",
                        )
                    with gr.Row():
                        hyp_num_trials = gr.Number(
                            label="Num Trials",
                            value=100,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_num_trials",
                            precision=0,
                        )
                        hyp_time_limit = gr.Number(
                            label="Time Limit",
                            value=3600,
                            visible=True,
                            interactive=True,
                            elem_id="hyp_time_limit",
                            precision=0,
                        )

        with gr.Row():
            add_job_button = gr.Button(value="Add Job", elem_id="add_job_button")
            clear_jobs_button = gr.Button(value="Clear Jobs", elem_id="clear_jobs_button")
            start_training_button = gr.Button(value="Start Training", elem_id="start_training_button")

        output_md = gr.Markdown(
            value="WARNING: Clicking `Start Training` will incur costs!", visible=True, interactive=False
        )
        jobs_df = gr.DataFrame(visible=False, interactive=False, value=pd.DataFrame())

        def _update_col_map(training_data):
            try:
                data_cols = pd.read_csv(training_data[0].name, nrows=2).columns.tolist()
            except TypeError:
                return [
                    gr.Dropdown.update(visible=True, interactive=False, choices=[], label="`id` column"),
                    gr.Dropdown.update(
                        visible=True,
                        interactive=False,
                        choices=[],
                        label="`target` column(s)",
                    ),
                ]
            return [
                gr.Dropdown.update(
                    visible=True,
                    interactive=True,
                    choices=[" "] + data_cols,
                    label="`id` column",
                    value="",
                ),
                gr.Dropdown.update(
                    visible=True,
                    interactive=True,
                    choices=data_cols,
                    label="`target` column(s)",
                    value=data_cols[1],
                ),
            ]

        training_data.change(
            _update_col_map,
            inputs=training_data,
            outputs=[col_map_id, col_map_target],
        )

        hyperparameters = [
            hyp_task,
            hyp_model,
            hyp_categorial_imputer,
            hyp_numerical_imputer,
            hyp_numeric_scaler,
            hyp_num_trials,
            hyp_time_limit,
        ]

        model_choice.change(
            app_utils.handle_model_choice_change,
            inputs=model_choice,
            outputs=[jobs_df, param_choice, autotrain_backend],
        )

        def _handle_param_choice_change(components):
            hyperparam_visibility = {}
            if components[param_choice] == "AutoTrain":
                for _hyperparameter in hyperparameters:
                    hyperparam_visibility[_hyperparameter.elem_id] = False
            else:
                for _hyperparameter in hyperparameters:
                    hyperparam_visibility[_hyperparameter.elem_id] = True
            op = [
                h.update(
                    interactive=hyperparam_visibility.get(h.elem_id, False),
                    visible=hyperparam_visibility.get(h.elem_id, False),
                )
                for h in hyperparameters
            ]
            op.append(jobs_df.update(value=pd.DataFrame(columns=[0]), visible=False, interactive=False))
            return op

        param_choice.change(
            _handle_param_choice_change,
            inputs=set([param_choice]),
            outputs=hyperparameters + [jobs_df],
        )

        def _add_job(components):
            try:
                _ = pd.read_csv(components[training_data][0].name, nrows=2).columns.tolist()
            except TypeError:
                raise gr.Error("Please upload training data first.")
            if len(components[col_map_target]) == 0:
                raise gr.Error("Target column cannot be empty.")
            if components[param_choice] == "AutoTrain" and components[autotrain_backend] != "AutoTrain":
                raise gr.Error("AutoTrain param choice is only available with AutoTrain backend.")

            _training_params = {}
            if components[param_choice] == "AutoTrain":
                for _hyperparameter in hyperparameters:
                    if _hyperparameter.elem_id in ["hyp_num_jobs", "hyp_language"]:
                        _training_params[_hyperparameter.elem_id] = components[_hyperparameter]
            else:
                for _hyperparameter in hyperparameters:
                    if _hyperparameter.elem_id not in ["hyp_num_jobs", "hyp_language"]:
                        _training_params[_hyperparameter.elem_id] = components[_hyperparameter]

            _training_params_df = app_utils.fetch_training_params_df(
                components[param_choice],
                components[jobs_df],
                _training_params,
                components[model_choice],
                components[autotrain_backend],
                hide_model_param=True,
            )
            return gr.DataFrame.update(value=_training_params_df, visible=True, interactive=False)

        add_job_button.click(
            _add_job,
            inputs=set(
                [
                    training_data,
                    param_choice,
                    autotrain_backend,
                    col_map_id,
                    col_map_target,
                    jobs_df,
                    model_choice,
                    autotrain_backend,
                ]
                + hyperparameters
            ),
            outputs=jobs_df,
        )

        clear_jobs_button.click(
            app_utils.clear_jobs,
            inputs=jobs_df,
            outputs=jobs_df,
        )

        start_training_button.click(
            start_training,
            inputs=[
                jobs_df,
                model_choice,
                training_data,
                validation_data,
                project_name,
                autotrain_username,
                user_token,
                col_map_id,
                col_map_target,
                hyp_task,
            ],
            outputs=output_md,
        )

        demo.load(
            app_utils._update_project_name,
            outputs=project_name,
        ).then(
            partial(app_utils._update_hub_model_choices, task="tabular"),
            outputs=model_choice,
        )

    return demo


if __name__ == "__main__":
    demo = main()
    demo.launch()



import argparse
import json
import os

import diffusers
import torch
import torch.nn.functional as F
import transformers
from accelerate import Accelerator
from accelerate.utils import ProjectConfiguration, set_seed
from diffusers import StableDiffusionXLPipeline
from diffusers.loaders import LoraLoaderMixin, text_encoder_lora_state_dict
from diffusers.models.attention_processor import (
    AttnAddedKVProcessor,
    AttnAddedKVProcessor2_0,
    LoRAAttnAddedKVProcessor,
    LoRAAttnProcessor,
    LoRAAttnProcessor2_0,
    SlicedAttnAddedKVProcessor,
)
from huggingface_hub import HfApi, snapshot_download

from autotrain import logger
from autotrain.trainers.dreambooth import utils
from autotrain.trainers.dreambooth.datasets import DreamBoothDataset, collate_fn
from autotrain.trainers.dreambooth.params import DreamBoothTrainingParams
from autotrain.trainers.dreambooth.trainer import Trainer
from autotrain.utils import monitor


def parse_args():
    # get training_config.json from the end user
    parser = argparse.ArgumentParser()
    parser.add_argument("--training_config", type=str, required=True)
    return parser.parse_args()


@monitor
def train(config):
    if isinstance(config, dict):
        config = DreamBoothTrainingParams(**config)
    config.prompt = str(config.prompt).strip()

    if config.model in utils.XL_MODELS:
        config.xl = True

    if config.repo_id is None and config.username is not None:
        config.repo_id = f"{config.username}/{config.project_name}"

    if config.project_name == "/tmp/model":
        snapshot_download(
            repo_id=config.image_path,
            local_dir=config.project_name,
            token=config.token,
            repo_type="dataset",
        )
        config.image_path = "/tmp/model/concept1/"

    accelerator_project_config = ProjectConfiguration(
        project_dir=config.project_name, logging_dir=os.path.join(config.project_name, "logs")
    )

    if config.fp16:
        mixed_precision = "fp16"
    elif config.bf16:
        mixed_precision = "bf16"
    else:
        mixed_precision = "no"

    accelerator = Accelerator(
        gradient_accumulation_steps=config.gradient_accumulation,
        mixed_precision=mixed_precision,
        log_with="tensorboard" if config.logging else None,
        project_config=accelerator_project_config,
    )

    if config.train_text_encoder and config.gradient_accumulation > 1 and accelerator.num_processes > 1:
        raise ValueError(
            "Gradient accumulation is not supported when training the text encoder in distributed training. "
            "Please set gradient_accumulation_steps to 1. This feature will be supported in the future."
        )

    if accelerator.is_local_main_process:
        transformers.utils.logging.set_verbosity_warning()
        diffusers.utils.logging.set_verbosity_info()
    else:
        transformers.utils.logging.set_verbosity_error()
        diffusers.utils.logging.set_verbosity_error()

    set_seed(config.seed)

    # Generate class images if prior preservation is enabled.
    if config.prior_preservation:
        utils.setup_prior_preservation(accelerator, config)

    # Handle the repository creation
    if accelerator.is_main_process:
        if config.project_name is not None:
            os.makedirs(config.project_name, exist_ok=True)

    weight_dtype = torch.float32
    if accelerator.mixed_precision == "fp16":
        weight_dtype = torch.float16
    elif accelerator.mixed_precision == "bf16":
        weight_dtype = torch.bfloat16
    tokenizers, text_encoders, vae, unet, noise_scheduler = utils.load_model_components(
        config, accelerator.device, weight_dtype
    )

    utils.enable_xformers(unet, config)
    utils.enable_gradient_checkpointing(unet, text_encoders, config)

    unet_lora_attn_procs = {}
    unet_lora_parameters = []
    for name, attn_processor in unet.attn_processors.items():
        cross_attention_dim = None if name.endswith("attn1.processor") else unet.config.cross_attention_dim
        if name.startswith("mid_block"):
            hidden_size = unet.config.block_out_channels[-1]
        elif name.startswith("up_blocks"):
            block_id = int(name[len("up_blocks.")])
            hidden_size = list(reversed(unet.config.block_out_channels))[block_id]
        elif name.startswith("down_blocks"):
            block_id = int(name[len("down_blocks.")])
            hidden_size = unet.config.block_out_channels[block_id]

        if isinstance(attn_processor, (AttnAddedKVProcessor, SlicedAttnAddedKVProcessor, AttnAddedKVProcessor2_0)):
            lora_attn_processor_class = LoRAAttnAddedKVProcessor
        else:
            lora_attn_processor_class = (
                LoRAAttnProcessor2_0 if hasattr(F, "scaled_dot_product_attention") else LoRAAttnProcessor
            )

        module = lora_attn_processor_class(hidden_size=hidden_size, cross_attention_dim=cross_attention_dim)
        unet_lora_attn_procs[name] = module
        unet_lora_parameters.extend(module.parameters())

    unet.set_attn_processor(unet_lora_attn_procs)

    text_lora_parameters = []
    if config.train_text_encoder:
        # ensure that dtype is float32, even if rest of the model that isn't trained is loaded in fp16
        text_lora_parameters = [
            LoraLoaderMixin._modify_text_encoder(_text_encoder, dtype=torch.float32) for _text_encoder in text_encoders
        ]

    def save_model_hook(models, weights, output_dir):
        # there are only two options here. Either are just the unet attn processor layers
        # or there are the unet and text encoder atten layers
        unet_lora_layers_to_save = None
        text_encoder_lora_layers_to_save = []

        for model in models:
            if isinstance(model, type(accelerator.unwrap_model(unet))):
                unet_lora_layers_to_save = utils.unet_attn_processors_state_dict(model)

            for _text_encoder in text_encoders:
                if isinstance(model, type(accelerator.unwrap_model(_text_encoder))):
                    text_encoder_lora_layers_to_save.append(text_encoder_lora_state_dict(model))

            # make sure to pop weight so that corresponding model is not saved again
            weights.pop()

        if len(text_encoder_lora_layers_to_save) == 0:
            LoraLoaderMixin.save_lora_weights(
                output_dir,
                unet_lora_layers=unet_lora_layers_to_save,
                text_encoder_lora_layers=None,
                safe_serialization=True,
            )
        elif len(text_encoder_lora_layers_to_save) == 1:
            LoraLoaderMixin.save_lora_weights(
                output_dir,
                unet_lora_layers=unet_lora_layers_to_save,
                text_encoder_lora_layers=text_encoder_lora_layers_to_save[0],
                safe_serialization=True,
            )
        elif len(text_encoder_lora_layers_to_save) == 2:
            StableDiffusionXLPipeline.save_lora_weights(
                output_dir,
                unet_lora_layers=unet_lora_layers_to_save,
                text_encoder_lora_layers=text_encoder_lora_layers_to_save[0],
                text_encoder_2_lora_layers=text_encoder_lora_layers_to_save[1],
                safe_serialization=True,
            )
        else:
            raise ValueError("unexpected number of text encoders")

    def load_model_hook(models, input_dir):
        unet_ = None
        text_encoders_ = []

        while len(models) > 0:
            model = models.pop()

            if isinstance(model, type(accelerator.unwrap_model(unet))):
                unet_ = model
            for _text_encoder in text_encoders:
                if isinstance(model, type(accelerator.unwrap_model(_text_encoder))):
                    text_encoders_.append(model)

        lora_state_dict, network_alpha = LoraLoaderMixin.lora_state_dict(input_dir)
        LoraLoaderMixin.load_lora_into_unet(lora_state_dict, network_alpha=network_alpha, unet=unet_)

        if len(text_encoders_) == 0:
            LoraLoaderMixin.load_lora_into_text_encoder(
                lora_state_dict,
                network_alpha=network_alpha,
                text_encoder=None,
            )
        elif len(text_encoders_) == 1:
            LoraLoaderMixin.load_lora_into_text_encoder(
                lora_state_dict,
                network_alpha=network_alpha,
                text_encoder=text_encoders_[0],
            )
        elif len(text_encoders_) == 2:
            LoraLoaderMixin.load_lora_into_text_encoder(
                lora_state_dict,
                network_alpha=network_alpha,
                text_encoder=text_encoders_[0],
            )
            LoraLoaderMixin.load_lora_into_text_encoder(
                lora_state_dict,
                network_alpha=network_alpha,
                text_encoder=text_encoders_[1],
            )
        else:
            raise ValueError("unexpected number of text encoders")

    accelerator.register_save_state_pre_hook(save_model_hook)
    accelerator.register_load_state_pre_hook(load_model_hook)

    if config.allow_tf32:
        torch.backends.cuda.matmul.allow_tf32 = True

    if config.scale_lr:
        config.lr = config.lr * config.gradient_accumulation * config.batch_size * accelerator.num_processes

    optimizer = utils.get_optimizer(config, unet_lora_parameters, text_lora_parameters)

    encoder_hs, instance_prompt_encoder_hs = utils.pre_compute_text_embeddings(
        config=config, text_encoders=text_encoders, tokenizers=tokenizers
    )
    train_dataset = DreamBoothDataset(
        config=config,
        tokenizers=tokenizers,
        encoder_hidden_states=encoder_hs,
        instance_prompt_encoder_hidden_states=instance_prompt_encoder_hs,
    )

    train_dataloader = torch.utils.data.DataLoader(
        train_dataset,
        batch_size=config.batch_size,
        shuffle=True,
        collate_fn=lambda examples: collate_fn(examples, config),
        num_workers=config.dataloader_num_workers,
    )
    trainer = Trainer(
        unet=unet,
        vae=vae,
        train_dataloader=train_dataloader,
        text_encoders=text_encoders,
        config=config,
        optimizer=optimizer,
        accelerator=accelerator,
        noise_scheduler=noise_scheduler,
        train_dataset=train_dataset,
        weight_dtype=weight_dtype,
        text_lora_parameters=text_lora_parameters,
        unet_lora_parameters=unet_lora_parameters,
        tokenizers=tokenizers,
    )
    trainer.train()

    # remove token key from training_params.json located in output directory
    # first check if file exists
    if os.path.exists(f"{config.project_name}/training_params.json"):
        training_params = json.load(open(f"{config.project_name}/training_params.json"))
        training_params.pop("token")
        json.dump(training_params, open(f"{config.project_name}/training_params.json", "w"))

    # remove config.image_path directory if it exists
    if os.path.exists(config.image_path):
        os.system(f"rm -rf {config.image_path}")

    # add config.prompt as a text file in the output directory
    with open(f"{config.project_name}/prompt.txt", "w") as f:
        f.write(config.prompt)

    if config.push_to_hub:
        trainer.push_to_hub()

    if "SPACE_ID" in os.environ:
        # shut down the space
        logger.info("Pausing space...")
        api = HfApi(token=config.token)
        api.pause_space(repo_id=os.environ["SPACE_ID"])


if __name__ == "__main__":
    args = parse_args()
    training_config = json.load(open(args.training_config))
    config = DreamBoothTrainingParams(**training_config)
    train(config)



import argparse
import json
import os
from functools import partial

import joblib
import numpy as np
import optuna
import pandas as pd
from datasets import load_dataset
from huggingface_hub import HfApi
from sklearn import pipeline, preprocessing
from sklearn.compose import ColumnTransformer

from autotrain import logger
from autotrain.trainers.tabular import utils
from autotrain.trainers.tabular.params import TabularParams
from autotrain.utils import monitor


def parse_args():
    # get training_config.json from the end user
    parser = argparse.ArgumentParser()
    parser.add_argument("--training_config", type=str, required=True)
    return parser.parse_args()


def optimize(trial, model_name, xtrain, xvalid, ytrain, yvalid, eval_metric, task, preprocessor):
    if isinstance(trial, dict):
        params = trial
    else:
        params = utils.get_params(trial, model_name, task)
    labels = None
    if task == "multi_class_classification":
        labels = np.unique(ytrain)
    metrics = utils.TabularMetrics(sub_task=task, labels=labels)

    if task in ("binary_classification", "multi_class_classification", "single_column_regression"):
        ytrain = ytrain.ravel()
        yvalid = yvalid.ravel()

    if preprocessor is not None:
        try:
            xtrain = preprocessor.fit_transform(xtrain)
            xvalid = preprocessor.transform(xvalid)
        except ValueError:
            logger.info("Preprocessing failed, using nan_to_num")
            train_cols = xtrain.columns.tolist()
            valid_cols = xvalid.columns.tolist()
            xtrain = np.nan_to_num(xtrain)
            xvalid = np.nan_to_num(xvalid)
            # convert back to dataframe
            xtrain = pd.DataFrame(xtrain, columns=train_cols)
            xvalid = pd.DataFrame(xvalid, columns=valid_cols)
            xtrain = preprocessor.fit_transform(xtrain)
            xvalid = preprocessor.transform(xvalid)

    if model_name == "xgboost":
        params["eval_metric"] = eval_metric

    _model = utils.TabularModel(model_name, preprocessor=None, sub_task=task, params=params)
    model = _model.pipeline
    models = []
    if task in ("multi_label_classification", "multi_column_regression"):
        # also multi_column_regression
        ypred = []
        models = [model] * ytrain.shape[1]
        for idx, _m in enumerate(models):
            if model_name == "xgboost":
                _m.fit(
                    xtrain,
                    ytrain[:, idx],
                    model__eval_set=[(xvalid, yvalid[:, idx])],
                    model__verbose=False,
                )
            else:
                _m.fit(xtrain, ytrain[:, idx])
            if task == "multi_column_regression":
                ypred_temp = _m.predict(xvalid)
            else:
                if _model.use_predict_proba:
                    ypred_temp = _m.predict_proba(xvalid)[:, 1]
                else:
                    ypred_temp = _m.predict(xvalid)
            ypred.append(ypred_temp)
        ypred = np.column_stack(ypred)

    else:
        models = [model]
        if model_name == "xgboost":
            model.fit(
                xtrain,
                ytrain,
                model__eval_set=[(xvalid, yvalid)],
                model__verbose=False,
            )
        else:
            models[0].fit(xtrain, ytrain)

        if _model.use_predict_proba:
            ypred = models[0].predict_proba(xvalid)
        else:
            ypred = models[0].predict(xvalid)

        if task == "multi_class_classification":
            if ypred.reshape(xvalid.shape[0], -1).shape[1] != len(labels):
                ypred_ohe = np.zeros((xvalid.shape[0], len(labels)))
                ypred_ohe[np.arange(xvalid.shape[0]), ypred] = 1
                ypred = ypred_ohe

        if task == "binary_classification":
            if ypred.reshape(xvalid.shape[0], -1).shape[1] != 2:
                ypred = np.column_stack([1 - ypred, ypred])

    # calculate metric
    metric_dict = metrics.calculate(yvalid, ypred)

    # change eval_metric key to loss
    if eval_metric in metric_dict:
        metric_dict["loss"] = metric_dict[eval_metric]

    logger.info(f"Metrics: {metric_dict}")
    if isinstance(trial, dict):
        return models, preprocessor, metric_dict
    return metric_dict["loss"]


@monitor
def train(config):
    if isinstance(config, dict):
        config = TabularParams(**config)

    if config.repo_id is None and config.username is not None:
        config.repo_id = f"{config.username}/{config.project_name}"

    logger.info("Starting training...")
    logger.info(f"Training config: {config}")

    train_data = None
    valid_data = None
    train_path = f"{config.data_path}/{config.train_split}.csv"
    if os.path.exists(train_path):
        logger.info("loading dataset from csv")
        train_data = pd.read_csv(train_path)
    else:
        train_data = load_dataset(
            config.data_path,
            split=config.train_split,
            token=config.token,
        )
        train_data = train_data.to_pandas()

    if config.valid_split is not None:
        valid_path = f"{config.data_path}/{config.valid_split}.csv"
        if os.path.exists(valid_path):
            logger.info("loading dataset from csv")
            valid_data = pd.read_csv(valid_path)
        else:
            valid_data = load_dataset(
                config.data_path,
                split=config.valid_split,
                token=config.token,
            )
            valid_data = valid_data.to_pandas()

    if valid_data is None:
        raise Exception("valid_data is None. Please provide a valid_split for tabular training.")

    # determine which columns are categorical
    if config.categorical_columns is None:
        config.categorical_columns = utils.get_categorical_columns(train_data)
    if config.numerical_columns is None:
        config.numerical_columns = utils.get_numerical_columns(train_data)

    _id_target_cols = (
        [config.id_column] + config.target_columns if config.id_column is not None else config.target_columns
    )
    config.numerical_columns = [c for c in config.numerical_columns if c not in _id_target_cols]
    config.categorical_columns = [c for c in config.categorical_columns if c not in _id_target_cols]

    useful_columns = config.categorical_columns + config.numerical_columns

    logger.info(f"Categorical columns: {config.categorical_columns}")
    logger.info(f"Numerical columns: {config.numerical_columns}")

    # convert object columns to categorical
    for col in config.categorical_columns:
        train_data[col] = train_data[col].astype("category")
        valid_data[col] = valid_data[col].astype("category")

    logger.info(f"Useful columns: {useful_columns}")

    target_encoders = {}
    if config.task == "classification":
        for target_column in config.target_columns:
            target_encoder = preprocessing.LabelEncoder()
            target_encoder.fit(train_data[target_column])
            target_encoders[target_column] = target_encoder

    # encode target columns in train and valid data
    for k, v in target_encoders.items():
        train_data.loc[:, k] = v.transform(train_data[k])
        valid_data.loc[:, k] = v.transform(valid_data[k])

    numeric_transformer = "passthrough"
    categorical_transformer = "passthrough"
    transformers = []
    preprocessor = None

    numeric_steps = []
    imputer = utils.get_imputer(config.numerical_imputer)
    scaler = utils.get_scaler(config.numeric_scaler)
    if imputer is not None:
        numeric_steps.append(("num_imputer", imputer))
    if scaler is not None:
        numeric_steps.append(("num_scaler", scaler))

    if len(numeric_steps) > 0:
        numeric_transformer = pipeline.Pipeline(numeric_steps)
        transformers.append(("numeric", numeric_transformer, config.numerical_columns))

    categorical_steps = []
    imputer = utils.get_imputer(config.categorical_imputer)
    if imputer is not None:
        categorical_steps.append(("cat_imputer", imputer))

    if len(config.categorical_columns) > 0:
        if config.model in ("xgboost", "lightgbm", "randomforest", "catboost", "extratrees"):
            categorical_steps.append(
                (
                    "cat_encoder",
                    preprocessing.OrdinalEncoder(
                        handle_unknown="use_encoded_value",
                        categories="auto",
                        unknown_value=np.nan,
                    ),
                )
            )
        else:
            categorical_steps.append(
                (
                    "cat_encoder",
                    preprocessing.OneHotEncoder(handle_unknown="ignore"),
                )
            )

    if len(categorical_steps) > 0:
        categorical_transformer = pipeline.Pipeline(categorical_steps)
        transformers.append(("categorical", categorical_transformer, config.categorical_columns))

    if len(transformers) > 0:
        preprocessor = ColumnTransformer(transformers=transformers, verbose=True, n_jobs=-1)
        logger.info(f"Preprocessor: {preprocessor}")

    xtrain = train_data[useful_columns].reset_index(drop=True)
    xvalid = valid_data[useful_columns].reset_index(drop=True)

    ytrain = train_data[config.target_columns].values
    yvalid = valid_data[config.target_columns].values

    # determine sub_task
    if config.task == "classification":
        if len(target_encoders) == 1:
            if len(target_encoders[config.target_columns[0]].classes_) == 2:
                sub_task = "binary_classification"
            else:
                sub_task = "multi_class_classification"
        else:
            sub_task = "multi_label_classification"
    else:
        if len(config.target_columns) > 1:
            sub_task = "multi_column_regression"
        else:
            sub_task = "single_column_regression"

    eval_metric, direction = utils.get_metric_direction(sub_task)

    args = {
        "model_name": config.model,
        "xtrain": xtrain,
        "xvalid": xvalid,
        "ytrain": ytrain,
        "yvalid": yvalid,
        "eval_metric": eval_metric,
        "task": sub_task,
        "preprocessor": preprocessor,
    }

    optimize_func = partial(optimize, **args)
    study = optuna.create_study(direction=direction, study_name="AutoTrain")
    study.optimize(optimize_func, n_trials=config.num_trials, timeout=config.time_limit)
    best_params = study.best_params

    logger.info(f"Best params: {best_params}")
    best_models, best_preprocessors, best_metrics = optimize(best_params, **args)

    models = (
        [pipeline.Pipeline([("preprocessor", best_preprocessors), ("model", m)]) for m in best_models]
        if best_preprocessors is not None
        else best_models
    )

    joblib.dump(
        models[0] if len(models) == 1 else models,
        os.path.join(config.project_name, "model.joblib"),
    )
    joblib.dump(target_encoders, os.path.join(config.project_name, "target_encoders.joblib"))

    model_card = utils.create_model_card(config, sub_task, best_params, best_metrics)

    if model_card is not None:
        with open(os.path.join(config.project_name, "README.md"), "w") as fp:
            fp.write(f"{model_card}")

    # remove token key from training_params.json located in output directory
    # first check if file exists
    if os.path.exists(f"{config.project_name}/training_params.json"):
        training_params = json.load(open(f"{config.project_name}/training_params.json"))
        training_params.pop("token")
        json.dump(training_params, open(f"{config.project_name}/training_params.json", "w"))

    # save model card to output directory as README.md
    with open(f"{config.project_name}/README.md", "w") as f:
        f.write(model_card)

    if config.push_to_hub:
        logger.info("Pushing model to hub...")
        api = HfApi(token=config.token)
        api.create_repo(repo_id=config.repo_id, repo_type="model", private=True)
        api.upload_folder(folder_path=config.project_name, repo_id=config.repo_id, repo_type="model")

    if "SPACE_ID" in os.environ:
        # shut down the space
        logger.info("Pausing space...")
        api = HfApi(token=config.token)
        api.pause_space(repo_id=os.environ["SPACE_ID"])

    if "ENDPOINT_ID" in os.environ:
        # shut down the endpoint
        logger.info("Pausing endpoint...")
        utils.pause_endpoint(config)


if __name__ == "__main__":
    args = parse_args()
    training_config = json.load(open(args.training_config))
    config = TabularParams(**training_config)
    train(config)



import argparse
import json
import os

from huggingface_hub import HfApi

from autotrain import logger
from autotrain.trainers.generic import utils
from autotrain.trainers.generic.params import GenericParams
from autotrain.utils import monitor


def parse_args():
    # get training_config.json from the end user
    parser = argparse.ArgumentParser()
    parser.add_argument("--config", type=str, required=True)
    return parser.parse_args()


@monitor
def run(config):
    if isinstance(config, dict):
        config = GenericParams(**config)

    # download the data repo
    logger.info("Downloading data repo...")
    utils.pull_dataset_repo(config)

    # install the requirements
    logger.info("Installing requirements...")
    utils.install_requirements(config)

    # run the command
    logger.info("Running command...")
    utils.run_command(config)

    if "SPACE_ID" in os.environ:
        # shut down the space
        logger.info("Pausing space...")
        api = HfApi(token=config.token)
        api.pause_space(repo_id=os.environ["SPACE_ID"])

    if "ENDPOINT_ID" in os.environ:
        # shut down the endpoint
        logger.info("Pausing endpoint...")
        utils.pause_endpoint(config)


if __name__ == "__main__":
    args = parse_args()
    _config = json.load(open(args.config))
    _config = GenericParams(**_config)
    run(_config)



import argparse
import json
import os

import pandas as pd
from accelerate.state import PartialState
from datasets import Dataset, load_dataset
from huggingface_hub import HfApi
from transformers import (
    AutoConfig,
    AutoModelForSequenceClassification,
    AutoTokenizer,
    EarlyStoppingCallback,
    Trainer,
    TrainingArguments,
)

from autotrain import logger
from autotrain.trainers.text_classification import utils
from autotrain.trainers.text_classification.dataset import TextClassificationDataset
from autotrain.trainers.text_classification.params import TextClassificationParams
from autotrain.utils import monitor


def parse_args():
    # get training_config.json from the end user
    parser = argparse.ArgumentParser()
    parser.add_argument("--training_config", type=str, required=True)
    return parser.parse_args()


@monitor
def train(config):
    if isinstance(config, dict):
        config = TextClassificationParams(**config)

    if config.repo_id is None and config.username is not None:
        config.repo_id = f"{config.username}/{config.project_name}"

    if PartialState().process_index == 0:
        logger.info("Starting training...")
        logger.info(f"Training config: {config}")

    train_data = None
    valid_data = None
    # check if config.train_split.csv exists in config.data_path
    if config.train_split is not None:
        train_path = f"{config.data_path}/{config.train_split}.csv"
        if os.path.exists(train_path):
            logger.info("loading dataset from csv")
            train_data = pd.read_csv(train_path)
            train_data = Dataset.from_pandas(train_data)
        else:
            train_data = load_dataset(
                config.data_path,
                split=config.train_split,
                token=config.token,
            )

    if config.valid_split is not None:
        valid_path = f"{config.data_path}/{config.valid_split}.csv"
        if os.path.exists(valid_path):
            logger.info("loading dataset from csv")
            valid_data = pd.read_csv(valid_path)
            valid_data = Dataset.from_pandas(valid_data)
        else:
            valid_data = load_dataset(
                config.data_path,
                split=config.valid_split,
                token=config.token,
            )

    classes = train_data.features[config.target_column].names
    label2id = {c: i for i, c in enumerate(classes)}
    num_classes = len(classes)

    if num_classes < 2:
        raise ValueError("Invalid number of classes. Must be greater than 1.")

    if config.valid_split is not None:
        num_classes_valid = len(valid_data.unique(config.target_column))
        if num_classes_valid != num_classes:
            raise ValueError(
                f"Number of classes in train and valid are not the same. Training has {num_classes} and valid has {num_classes_valid}"
            )

    model_config = AutoConfig.from_pretrained(config.model, num_labels=num_classes)
    model_config._num_labels = len(label2id)
    model_config.label2id = label2id
    model_config.id2label = {v: k for k, v in label2id.items()}

    try:
        model = AutoModelForSequenceClassification.from_pretrained(
            config.model, config=model_config, trust_remote_code=True, token=config.token
        )
    except OSError:
        model = AutoModelForSequenceClassification.from_pretrained(
            config.model, config=model_config, from_tf=True, trust_remote_code=True, token=config.token
        )

    tokenizer = AutoTokenizer.from_pretrained(config.model, token=config.token, trust_remote_code=True)
    train_data = TextClassificationDataset(data=train_data, tokenizer=tokenizer, config=config)
    if config.valid_split is not None:
        valid_data = TextClassificationDataset(data=valid_data, tokenizer=tokenizer, config=config)

    if config.logging_steps == -1:
        if config.valid_split is not None:
            logging_steps = int(0.2 * len(valid_data) / config.batch_size)
        else:
            logging_steps = int(0.2 * len(train_data) / config.batch_size)
        if logging_steps == 0:
            logging_steps = 1

    else:
        logging_steps = config.logging_steps

    training_args = dict(
        output_dir=config.project_name,
        per_device_train_batch_size=config.batch_size,
        per_device_eval_batch_size=2 * config.batch_size,
        learning_rate=config.lr,
        num_train_epochs=config.epochs,
        fp16=config.fp16,
        evaluation_strategy=config.evaluation_strategy if config.valid_split is not None else "no",
        logging_steps=logging_steps,
        save_total_limit=config.save_total_limit,
        save_strategy=config.save_strategy,
        gradient_accumulation_steps=config.gradient_accumulation,
        report_to=config.log,
        auto_find_batch_size=config.auto_find_batch_size,
        lr_scheduler_type=config.scheduler,
        optim=config.optimizer,
        warmup_ratio=config.warmup_ratio,
        weight_decay=config.weight_decay,
        max_grad_norm=config.max_grad_norm,
        push_to_hub=False,
        load_best_model_at_end=True if config.valid_split is not None else False,
        ddp_find_unused_parameters=False,
    )

    if config.valid_split is not None:
        early_stop = EarlyStoppingCallback(early_stopping_patience=3, early_stopping_threshold=0.01)
        callbacks_to_use = [early_stop]
    else:
        callbacks_to_use = []

    args = TrainingArguments(**training_args)
    trainer_args = dict(
        args=args,
        model=model,
        callbacks=callbacks_to_use,
        compute_metrics=utils._binary_classification_metrics
        if num_classes == 2
        else utils._multi_class_classification_metrics,
    )

    trainer = Trainer(
        **trainer_args,
        train_dataset=train_data,
        eval_dataset=valid_data,
    )
    trainer.train()

    logger.info("Finished training, saving model...")
    trainer.save_model(config.project_name)
    tokenizer.save_pretrained(config.project_name)

    model_card = utils.create_model_card(config, trainer, num_classes)

    # remove token key from training_params.json located in output directory
    # first check if file exists
    if os.path.exists(f"{config.project_name}/training_params.json"):
        training_params = json.load(open(f"{config.project_name}/training_params.json"))
        training_params.pop("token")
        json.dump(training_params, open(f"{config.project_name}/training_params.json", "w"))

    # save model card to output directory as README.md
    with open(f"{config.project_name}/README.md", "w") as f:
        f.write(model_card)

    if config.push_to_hub:
        if PartialState().process_index == 0:
            logger.info("Pushing model to hub...")
            api = HfApi(token=config.token)
            api.create_repo(repo_id=config.repo_id, repo_type="model", private=True)
            api.upload_folder(folder_path=config.project_name, repo_id=config.repo_id, repo_type="model")

    if PartialState().process_index == 0:
        if "SPACE_ID" in os.environ:
            # shut down the space
            logger.info("Pausing space...")
            api = HfApi(token=config.token)
            api.pause_space(repo_id=os.environ["SPACE_ID"])

        if "ENDPOINT_ID" in os.environ:
            # shut down the endpoint
            logger.info("Pausing endpoint...")
            utils.pause_endpoint(config)


if __name__ == "__main__":
    args = parse_args()
    training_config = json.load(open(args.training_config))
    config = TextClassificationParams(**training_config)
    train(config)



import os

from pydantic import BaseModel

from autotrain import logger


class AutoTrainParams(BaseModel):
    def save(self, output_dir):
        os.makedirs(output_dir, exist_ok=True)
        path = os.path.join(output_dir, "training_params.json")
        # save formatted json
        with open(path, "w") as f:
            f.write(self.json(indent=4))

    def __str__(self):
        data = self.dict()
        data["token"] = "*****" if data.get("token") else None
        return str(data)

    def __init__(self, **data):
        super().__init__(**data)

        # Parameters not supplied by the user
        defaults = {f.name for f in self.__fields__.values() if f.default == self.__dict__[f.name]}
        supplied = set(data.keys())
        not_supplied = defaults - supplied
        if not_supplied:
            logger.warning(f"Parameters not supplied by user and set to default: {', '.join(not_supplied)}")

        # Parameters that were supplied but not used
        # This is a naive implementation. It might catch some internal Pydantic params.
        unused = supplied - set(self.__fields__)
        if unused:
            logger.warning(f"Parameters supplied but not used: {', '.join(unused)}")



from typing import Dict

from pydantic import Field

from autotrain.trainers.common import AutoTrainParams


class GenericParams(AutoTrainParams):
    username: str = Field(None, title="Hugging Face Username")
    project_name: str = Field(None, title="Output directory")
    data_path: str = Field(None, title="Data path")
    token: str = Field(None, title="Hub Token")
    script_path: str = Field(None, title="Script path")
    env: Dict[str, str] = Field(None, title="Environment Variables")



import io
import json
import os
from dataclasses import dataclass
from typing import Union

import requests
from huggingface_hub import HfApi

from autotrain import logger
from autotrain.dataset import AutoTrainDataset, AutoTrainDreamboothDataset
from autotrain.trainers.clm.params import LLMTrainingParams
from autotrain.trainers.dreambooth.params import DreamBoothTrainingParams
from autotrain.trainers.generic.params import GenericParams
from autotrain.trainers.image_classification.params import ImageClassificationParams
from autotrain.trainers.tabular.params import TabularParams
from autotrain.trainers.text_classification.params import TextClassificationParams


_DOCKERFILE = """
FROM huggingface/autotrain-advanced:latest

CMD autotrain api --port 7860 --host 0.0.0.0
"""

# format _DOCKERFILE
_DOCKERFILE = _DOCKERFILE.replace("\n", " ").replace("  ", "\n").strip()


def _tabular_munge_data(params, username):
    if isinstance(params.target_columns, str):
        col_map_label = [params.target_columns]
    else:
        col_map_label = params.target_columns
    task = params.task
    if task == "classification" and len(col_map_label) > 1:
        task = "tabular_multi_label_classification"
    elif task == "classification" and len(col_map_label) == 1:
        task = "tabular_multi_class_classification"
    elif task == "regression" and len(col_map_label) > 1:
        task = "tabular_multi_column_regression"
    elif task == "regression" and len(col_map_label) == 1:
        task = "tabular_single_column_regression"
    else:
        raise Exception("Please select a valid task.")

    train_data_path = f"{params.data_path}/{params.train_split}.csv"
    if params.valid_split is not None:
        valid_data_path = f"{params.data_path}/{params.valid_split}.csv"
    else:
        valid_data_path = []
    if os.path.exists(train_data_path):
        dset = AutoTrainDataset(
            train_data=[train_data_path],
            task=task,
            token=params.token,
            project_name=params.project_name,
            username=username,
            column_mapping={"id": params.col_map_id, "label": col_map_label},
            valid_data=valid_data_path,
            percent_valid=None,  # TODO: add to UI
        )
        dset.prepare()
        return f"{username}/autotrain-data-{params.project_name}"

    return params.data_path


def _llm_munge_data(params, username):
    train_data_path = f"{params.data_path}/{params.train_split}.csv"
    if params.valid_split is not None:
        valid_data_path = f"{params.data_path}/{params.valid_split}.csv"
    else:
        valid_data_path = []
    if os.path.exists(train_data_path):
        dset = AutoTrainDataset(
            train_data=[train_data_path],
            task="lm_training",
            token=params.token,
            project_name=params.project_name,
            username=username,
            column_mapping={"text": params.text_column},
            valid_data=valid_data_path,
            percent_valid=None,  # TODO: add to UI
        )
        dset.prepare()
        return f"{username}/autotrain-data-{params.project_name}"

    return params.data_path


def _text_clf_munge_data(params, username):
    train_data_path = f"{params.data_path}/{params.train_split}.csv"
    if params.valid_split is not None:
        valid_data_path = f"{params.data_path}/{params.valid_split}.csv"
    else:
        valid_data_path = None
    if os.path.exists(train_data_path):
        dset = AutoTrainDataset(
            train_data=[train_data_path],
            valid_data=[valid_data_path] if valid_data_path is not None else None,
            task="text_multi_class_classification",
            token=params.token,
            project_name=params.project_name,
            username=username,
            column_mapping={"text": params.text_column, "label": params.target_column},
            percent_valid=None,  # TODO: add to UI
        )
        dset.prepare()
        return f"{username}/autotrain-data-{params.project_name}"

    return params.data_path


def _dreambooth_munge_data(params, username):
    # check if params.image_path is a directory
    if os.path.isdir(params.image_path):
        training_data = [os.path.join(params.image_path, f) for f in os.listdir(params.image_path)]
        training_data = [io.BytesIO(open(f, "rb").read()) for f in training_data]
        dset = AutoTrainDreamboothDataset(
            concept_images=training_data,
            concept_name=params.prompt,
            token=params.token,
            project_name=params.project_name,
            username=username,
        )
        dset.prepare()
        return f"{username}/autotrain-data-{params.project_name}"
    return params.image_path


@dataclass
class EndpointsRunner:
    params: Union[TextClassificationParams, ImageClassificationParams, LLMTrainingParams]
    backend: str

    def __post_init__(self):
        self.endpoints_backends = {
            "ep-aws-useast1-s": "aws_us-east-1_gpu_small_g4dn.xlarge",
            "ep-aws-useast1-m": "aws_us-east-1_gpu_medium_g5.2xlarge",
            "ep-aws-useast1-l": "aws_us-east-1_gpu_large_g4dn.12xlarge",
            "ep-aws-useast1-xl": "aws_us-east-1_gpu_xlarge_p4de",
            "ep-aws-useast1-2xl": "aws_us-east-1_gpu_2xlarge_p4de",
            "ep-aws-useast1-4xl": "aws_us-east-1_gpu_4xlarge_p4de",
            "ep-aws-useast1-8xl": "aws_us-east-1_gpu_8xlarge_p4de",
        }
        if self.params.repo_id is not None:
            self.username = self.params.repo_id.split("/")[0]
        elif self.params.username is not None:
            self.username = self.params.username
        else:
            raise ValueError("Must provide either repo_id or username")
        self.api_url = f"https://api.endpoints.huggingface.cloud/v2/endpoint/{self.username}"
        if isinstance(self.params, LLMTrainingParams):
            self.task_id = 9

    def _create_endpoint(self):
        hardware = self.endpoints_backends[self.backend]
        accelerator = hardware.split("_")[2]
        instance_size = hardware.split("_")[3]
        region = hardware.split("_")[1]
        vendor = hardware.split("_")[0]
        instance_type = hardware.split("_")[4]
        payload = {
            "accountId": self.username,
            "compute": {
                "accelerator": accelerator,
                "instanceSize": instance_size,
                "instanceType": instance_type,
                "scaling": {"maxReplica": 1, "minReplica": 1},
            },
            "model": {
                "framework": "custom",
                "image": {
                    "custom": {
                        "env": {
                            "HF_TOKEN": self.params.token,
                            "AUTOTRAIN_USERNAME": self.username,
                            "PROJECT_NAME": self.params.project_name,
                            "PARAMS": json.dumps(self.params.json()),
                            "DATA_PATH": self.params.data_path,
                            "TASK_ID": str(self.task_id),
                            "MODEL": self.params.model,
                            "OUTPUT_MODEL_REPO": self.params.repo_id,
                            "ENDPOINT_ID": f"{self.username}/{self.params.project_name}",
                        },
                        "health_route": "/",
                        "port": 7860,
                        "url": "huggingface/autotrain-advanced-api:latest",
                    }
                },
                "repository": "autotrain-projects/autotrain-advanced",
                "revision": "main",
                "task": "custom",
            },
            "name": self.params.project_name,
            "provider": {"region": region, "vendor": vendor},
            "type": "protected",
        }
        headers = {"Authorization": f"Bearer {self.params.token}"}
        r = requests.post(self.api_url, json=payload, headers=headers)
        logger.info(r.json())
        return r.json()

    def prepare(self):
        if isinstance(self.params, LLMTrainingParams):
            data_path = _llm_munge_data(self.params, self.username)
            self.params.data_path = data_path
            endpoint_id = self._create_endpoint()
            return endpoint_id
        if isinstance(self.params, TextClassificationParams):
            data_path = _text_clf_munge_data(self.params, self.username)
            self.params.data_path = data_path
            endpoint_id = self._create_endpoint()
            return endpoint_id
        raise NotImplementedError


@dataclass
class SpaceRunner:
    params: Union[TextClassificationParams, ImageClassificationParams, LLMTrainingParams, GenericParams, TabularParams]
    backend: str

    def __post_init__(self):
        self.spaces_backends = {
            "a10gl": "a10g-large",
            "a10gs": "a10g-small",
            "a100": "a100-large",
            "t4m": "t4-medium",
            "t4s": "t4-small",
            "cpu": "cpu-upgrade",
            "cpuf": "cpu-basic",
        }
        if not isinstance(self.params, GenericParams):
            if self.params.repo_id is not None:
                self.username = self.params.repo_id.split("/")[0]
            elif self.params.username is not None:
                self.username = self.params.username
            else:
                raise ValueError("Must provide either repo_id or username")
        else:
            self.username = self.params.username

        if isinstance(self.params, LLMTrainingParams):
            self.task_id = 9
        elif isinstance(self.params, TextClassificationParams):
            self.task_id = 2
        elif isinstance(self.params, TabularParams):
            self.task_id = 26
        elif isinstance(self.params, GenericParams):
            self.task_id = 27
        elif isinstance(self.params, DreamBoothTrainingParams):
            self.task_id = 25
        else:
            raise NotImplementedError

    def prepare(self):
        if isinstance(self.params, LLMTrainingParams):
            self.task_id = 9
            data_path = _llm_munge_data(self.params, self.username)
            self.params.data_path = data_path
            space_id = self._create_space()
            return space_id
        if isinstance(self.params, TextClassificationParams):
            self.task_id = 2
            data_path = _text_clf_munge_data(self.params, self.username)
            self.params.data_path = data_path
            space_id = self._create_space()
            return space_id
        if isinstance(self.params, TabularParams):
            self.task_id = 26
            data_path = _tabular_munge_data(self.params, self.username)
            self.params.data_path = data_path
            space_id = self._create_space()
            return space_id
        if isinstance(self.params, GenericParams):
            self.task_id = 27
            space_id = self._create_space()
            return space_id
        if isinstance(self.params, DreamBoothTrainingParams):
            self.task_id = 25
            data_path = _dreambooth_munge_data(self.params, self.username)
            space_id = self._create_space()
            return space_id
        raise NotImplementedError

    def _create_readme(self):
        _readme = "---\n"
        _readme += f"title: {self.params.project_name}\n"
        _readme += "emoji: 🚀\n"
        _readme += "colorFrom: green\n"
        _readme += "colorTo: indigo\n"
        _readme += "sdk: docker\n"
        _readme += "pinned: false\n"
        _readme += "duplicated_from: autotrain-projects/autotrain-advanced\n"
        _readme += "---\n"
        _readme = io.BytesIO(_readme.encode())
        return _readme

    def _add_secrets(self, api, repo_id):
        if isinstance(self.params, GenericParams):
            for k, v in self.params.env.items():
                api.add_space_secret(repo_id=repo_id, key=k, value=v)
            self.params.env = {}

        api.add_space_secret(repo_id=repo_id, key="HF_TOKEN", value=self.params.token)
        api.add_space_secret(repo_id=repo_id, key="AUTOTRAIN_USERNAME", value=self.username)
        api.add_space_secret(repo_id=repo_id, key="PROJECT_NAME", value=self.params.project_name)
        api.add_space_secret(repo_id=repo_id, key="TASK_ID", value=str(self.task_id))
        api.add_space_secret(repo_id=repo_id, key="PARAMS", value=json.dumps(self.params.json()))

        if isinstance(self.params, DreamBoothTrainingParams):
            api.add_space_secret(repo_id=repo_id, key="DATA_PATH", value=self.params.image_path)
        else:
            api.add_space_secret(repo_id=repo_id, key="DATA_PATH", value=self.params.data_path)

        if not isinstance(self.params, GenericParams):
            api.add_space_secret(repo_id=repo_id, key="MODEL", value=self.params.model)
            api.add_space_secret(repo_id=repo_id, key="OUTPUT_MODEL_REPO", value=self.params.repo_id)

    def _create_space(self):
        api = HfApi(token=self.params.token)
        repo_id = f"{self.username}/autotrain-{self.params.project_name}"
        api.create_repo(
            repo_id=repo_id,
            repo_type="space",
            space_sdk="docker",
            space_hardware=self.spaces_backends[self.backend.split("-")[1].lower()],
            private=True,
        )
        self._add_secrets(api, repo_id)
        readme = self._create_readme()
        api.upload_file(
            path_or_fileobj=readme,
            path_in_repo="README.md",
            repo_id=repo_id,
            repo_type="space",
        )

        _dockerfile = io.BytesIO(_DOCKERFILE.encode())
        api.upload_file(
            path_or_fileobj=_dockerfile,
            path_in_repo="Dockerfile",
            repo_id=repo_id,
            repo_type="space",
        )
        return repo_id



import copy
import random
import string

import gradio as gr
import numpy as np
import pandas as pd
from huggingface_hub import list_models

from autotrain import logger
from autotrain.utils import user_authentication


THEME = "freddyaboulton/dracula_revamped"

BACKEND_CHOICES = {
    "A10G Large": 3.15,
    "A10G Small": 1.05,
    "A100 Large": 4.13,
    "T4 Medium": 0.9,
    "T4 Small": 0.6,
    "CPU Upgrade": 0.03,
    "CPU (Free)": 0.0,
    # "Local": 0.0,
    # "AutoTrain": -1,
}


def estimate_cost():
    pass


def _update_hub_model_choices(task):
    logger.info(f"Updating hub model choices for task: {task}")
    if task == "text_multi_class_classification":
        hub_models1 = list_models(filter="fill-mask", sort="downloads", direction=-1, limit=100)
        hub_models2 = list_models(filter="text-classification", sort="downloads", direction=-1, limit=100)
        hub_models = list(hub_models1) + list(hub_models2)
    elif task == "lm_training":
        hub_models = list(list_models(filter="text-generation", sort="downloads", direction=-1, limit=100))
    elif task == "image_multi_class_classification":
        hub_models = list(list_models(filter="image-classification", sort="downloads", direction=-1, limit=100))
    elif task == "dreambooth":
        hub_models = list(list_models(filter="text-to-image", sort="downloads", direction=-1, limit=100))
    elif task == "tabular":
        return gr.Dropdown.update(
            choices=[],
            visible=False,
            interactive=False,
        )
    else:
        raise NotImplementedError
    # sort by number of downloads in descending order
    hub_models = [{"id": m.modelId, "downloads": m.downloads} for m in hub_models if m.private is False]
    hub_models = sorted(hub_models, key=lambda x: x["downloads"], reverse=True)

    if task == "dreambooth":
        choices = [
            "stabilityai/stable-diffusion-xl-base-1.0",
            "runwayml/stable-diffusion-v1-5",
            "stabilityai/stable-diffusion-2-1",
            "stabilityai/stable-diffusion-2-1-base",
        ]
        value = choices[0]
        return gr.Dropdown.update(
            choices=choices,
            value=value,
            visible=True,
            interactive=True,
        )

    # if task in ("text_multi_class_classification", "image_multi_class_classification"):
    #     choices = ["AutoTrain"] + [m["id"] for m in hub_models]
    # else:
    choices = [m["id"] for m in hub_models]

    return gr.Dropdown.update(
        choices=choices,
        value=choices[0],
        visible=True,
        interactive=True,
    )


def _update_project_name():
    random_project_name = "-".join(
        ["".join(random.choices(string.ascii_lowercase + string.digits, k=4)) for _ in range(3)]
    )
    return gr.Text.update(value=random_project_name, visible=True, interactive=True)


def _login_user(user_token):
    user_info = user_authentication(token=user_token)
    username = user_info["name"]

    user_can_pay = user_info["canPay"]
    orgs = user_info["orgs"]

    valid_orgs = [org for org in orgs if org["canPay"] is True]
    valid_orgs = [org for org in valid_orgs if org["roleInOrg"] in ("admin", "write")]
    valid_orgs = [org["name"] for org in valid_orgs]

    valid_can_pay = [username] + valid_orgs if user_can_pay else valid_orgs
    who_is_training = [username] + [org["name"] for org in orgs]
    return user_token, valid_can_pay, who_is_training


def fetch_training_params_df(
    param_choice, jobs_df, training_params, model_choice, autotrain_backend, hide_model_param=False
):
    if param_choice == "AutoTrain":
        # create a new dataframe from dict
        _training_params_df = pd.DataFrame([training_params])
    else:
        # add row to the dataframe
        if len(jobs_df) == 0:
            _training_params_df = pd.DataFrame([training_params])
        else:
            _training_params_df = copy.deepcopy(jobs_df)
            _training_params_df.columns = [f"hyp_{c}" for c in _training_params_df.columns]
            # convert dataframe to list of dicts
            _training_params_df = _training_params_df.to_dict(orient="records")
            # append new dict to the list
            _training_params_df.append(training_params)
            _training_params_df = pd.DataFrame(_training_params_df)
            # drop rows with all nan values
            _training_params_df.replace("", np.nan, inplace=True)
            # Drop rows with all missing values
            _training_params_df = _training_params_df.dropna(how="all")
            # Drop columns with all missing values
            _training_params_df = _training_params_df.dropna(axis=1, how="all")

    # remove hyp_ from column names
    _training_params_df.columns = [c[len("hyp_") :] for c in _training_params_df.columns]
    _training_params_df = _training_params_df.reset_index(drop=True)
    if not hide_model_param:
        _training_params_df.loc[:, "model_choice"] = model_choice
        _training_params_df.loc[:, "param_choice"] = param_choice
    _training_params_df.loc[:, "backend"] = autotrain_backend
    return _training_params_df


def clear_jobs(jobs_df):
    return gr.DataFrame.update(visible=False, interactive=False, value=pd.DataFrame())


def handle_model_choice_change(model_choice):
    op = []
    op.append(gr.DataFrame.update(value=pd.DataFrame(), visible=False, interactive=False))
    if model_choice == "AutoTrain":
        op.append(gr.Dropdown.update(value="AutoTrain", interactive=False))
        op.append(gr.Dropdown.update(value="AutoTrain", interactive=False))
    else:
        op.append(gr.Dropdown.update(value="Manual", interactive=True))
        op.append(gr.Dropdown.update(value=list(BACKEND_CHOICES.keys())[0], interactive=True))
    return op



import os

import gradio as gr

from autotrain import allowed_file_types
from autotrain.apps.utils import BACKEND_CHOICES, _login_user
from autotrain.utils import get_user_token


def user_validation():
    user_token = os.environ.get("HF_TOKEN", "")

    if len(user_token) == 0:
        user_token = get_user_token()

    if user_token is None:
        gr.Markdown(
            """Please login with a write [token](https://huggingface.co/settings/tokens).
            Pass your HF token in an environment variable called `HF_TOKEN` and then restart this app.
            """
        )
        raise ValueError("Please login with a write token.")

    user_token, valid_can_pay, who_is_training = _login_user(user_token)

    if user_token is None or len(user_token) == 0:
        gr.Error("Please login with a write token.")

    user_token = gr.Textbox(value=user_token, type="password", lines=1, max_lines=1, visible=False, interactive=False)
    valid_can_pay = gr.Textbox(value=",".join(valid_can_pay), visible=False, interactive=False)

    return user_token, valid_can_pay, who_is_training


def base_components(who_is_training):
    with gr.Row():
        with gr.Group():
            with gr.Column():
                with gr.Row():
                    autotrain_username = gr.Dropdown(
                        label="AutoTrain Username",
                        choices=who_is_training,
                        value=who_is_training[0] if who_is_training else "",
                        interactive=True,
                    )
                with gr.Row():
                    project_name = gr.Textbox(
                        label="Project name",
                        value="",
                        lines=1,
                        max_lines=1,
                        interactive=True,
                    )
                    model_choice = gr.Dropdown(
                        label="Model Choice",
                        choices=[],
                        visible=True,
                        interactive=True,
                        allow_custom_value=True,
                    )
                    autotrain_backend = gr.Dropdown(
                        label="Backend",
                        choices=list(BACKEND_CHOICES.keys()),
                        value=list(BACKEND_CHOICES.keys())[0],
                        interactive=True,
                    )

    return autotrain_username, project_name, model_choice, autotrain_backend


def train_valid_components():
    with gr.Column():
        with gr.Tabs(elem_id="tabs"):
            with gr.TabItem("Training Data"):
                training_data = gr.File(
                    label="Training Data",
                    file_types=allowed_file_types.TEXT_CLASSIFICATION,
                    file_count="multiple",
                    visible=True,
                    interactive=True,
                    elem_id="training_data",
                )
            with gr.TabItem("Validation Data (Optional)"):
                validation_data = gr.File(
                    label="Validation Data",
                    file_types=allowed_file_types.TEXT_CLASSIFICATION,
                    file_count="multiple",
                    visible=True,
                    interactive=True,
                    elem_id="validation_data",
                )
    return training_data, validation_data



import json
import os
import subprocess

import psutil
from fastapi import FastAPI

from autotrain import logger
from autotrain.trainers.clm.params import LLMTrainingParams
from autotrain.trainers.dreambooth.params import DreamBoothTrainingParams
from autotrain.trainers.generic.params import GenericParams
from autotrain.trainers.tabular.params import TabularParams
from autotrain.trainers.text_classification.params import TextClassificationParams


HF_TOKEN = os.environ.get("HF_TOKEN")
AUTOTRAIN_USERNAME = os.environ.get("AUTOTRAIN_USERNAME")
PROJECT_NAME = os.environ.get("PROJECT_NAME")
TASK_ID = int(os.environ.get("TASK_ID"))
PARAMS = os.environ.get("PARAMS")
DATA_PATH = os.environ.get("DATA_PATH")
MODEL = os.environ.get("MODEL")
OUTPUT_MODEL_REPO = os.environ.get("OUTPUT_MODEL_REPO")
PID = None


api = FastAPI()
logger.info(f"AUTOTRAIN_USERNAME: {AUTOTRAIN_USERNAME}")
logger.info(f"PROJECT_NAME: {PROJECT_NAME}")
logger.info(f"TASK_ID: {TASK_ID}")
logger.info(f"DATA_PATH: {DATA_PATH}")
logger.info(f"MODEL: {MODEL}")
logger.info(f"OUTPUT_MODEL_REPO: {OUTPUT_MODEL_REPO}")


def run_training():
    params = json.loads(PARAMS)
    logger.info(params)
    if TASK_ID == 9:
        params = LLMTrainingParams.parse_raw(params)
        params.project_name = "/tmp/model"
        params.save(output_dir=params.project_name)
        cmd = ["accelerate", "launch", "--num_machines", "1", "--num_processes", "1"]
        cmd.append("--mixed_precision")
        if params.fp16:
            cmd.append("fp16")
        else:
            cmd.append("no")

        cmd.extend(
            [
                "-m",
                "autotrain.trainers.clm",
                "--training_config",
                os.path.join(params.project_name, "training_params.json"),
            ]
        )
    elif TASK_ID in (1, 2):
        params = TextClassificationParams.parse_raw(params)
        params.project_name = "/tmp/model"
        params.save(output_dir=params.project_name)
        cmd = ["accelerate", "launch", "--num_machines", "1", "--num_processes", "1"]
        cmd.append("--mixed_precision")
        if params.fp16:
            cmd.append("fp16")
        else:
            cmd.append("no")

        cmd.extend(
            [
                "-m",
                "autotrain.trainers.text_classification",
                "--training_config",
                os.path.join(params.project_name, "training_params.json"),
            ]
        )
    elif TASK_ID in (13, 14, 15, 16, 26):
        params = TabularParams.parse_raw(params)
        params.project_name = "/tmp/model"
        params.save(output_dir=params.project_name)
        cmd = [
            "python",
            "-m",
            "autotrain.trainers.tabular",
            "--training_config",
            os.path.join(params.project_name, "training_params.json"),
        ]
    elif TASK_ID == 27:
        params = GenericParams.parse_raw(params)
        params.project_name = "/tmp/model"
        params.save(output_dir=params.project_name)
        cmd = [
            "python",
            "-m",
            "autotrain.trainers.generic",
            "--config",
            os.path.join(params.project_name, "training_params.json"),
        ]
    elif TASK_ID == 25:
        params = DreamBoothTrainingParams.parse_raw(params)
        params.project_name = "/tmp/model"
        params.save(output_dir=params.project_name)
        cmd = [
            "python",
            "-m",
            "autotrain.trainers.dreambooth",
            "--training_config",
            os.path.join(params.project_name, "training_params.json"),
        ]

    else:
        raise NotImplementedError

    cmd = [str(c) for c in cmd]
    logger.info(cmd)
    env = os.environ.copy()
    process = subprocess.Popen(" ".join(cmd), shell=True, env=env)
    return process.pid


def get_process_status(pid):
    try:
        process = psutil.Process(pid)
        return process.status()
    except psutil.NoSuchProcess:
        return "No process found with PID: {}".format(pid)


def kill_process(pid):
    try:
        parent_process = psutil.Process(pid)
        children = parent_process.children(recursive=True)  # This will get all the child processes recursively

        # First, terminate the child processes
        for child in children:
            child.terminate()

        # Wait for the child processes to terminate, and kill them if they don't
        gone, still_alive = psutil.wait_procs(children, timeout=3)
        for child in still_alive:
            child.kill()

        # Now, terminate the parent process
        parent_process.terminate()
        parent_process.wait(timeout=5)

        logger.info(f"Process with pid {pid} and its children have been killed")
        return f"Process with pid {pid} and its children have been killed"

    except psutil.NoSuchProcess:
        logger.info(f"No process found with pid {pid}")
        return f"No process found with pid {pid}"

    except psutil.TimeoutExpired:
        logger.info(f"Process {pid} or one of its children has not terminated in time")
        return f"Process {pid} or one of its children has not terminated in time"


@api.on_event("startup")
async def startup_event():
    process_pid = run_training()
    logger.info(f"Started training with PID {process_pid}")
    global PID
    PID = process_pid


@api.get("/")
async def root():
    return "Your model is being trained..."


@api.get("/status")
async def status():
    return get_process_status(pid=PID)


@api.get("/kill")
async def kill():
    return kill_process(pid=PID)


@api.get("/health")
async def health():
    return "OK"



import sys

from accelerate.state import PartialState
from loguru import logger


emojis = {
    "TRACE": "🔍",
    "DEBUG": "🐛",
    "INFO": "🚀",
    "SUCCESS": "✅",
    "WARNING": "⚠️",
    "ERROR": "❌",
    "CRITICAL": "🚨",
}


def should_log(record):
    return PartialState().is_main_process


def emoji_filter(record):
    level = record["level"].name
    record["level_emoji"] = emojis.get(level, "") + " " + level
    return True


log_format = (
    "<level>{level_emoji: <8}</level> | "
    "<green>{time:YYYY-MM-DD HH:mm:ss}</green> | "
    "<cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - "
    "<level>{message}</level>"
)

logger.remove()
if not hasattr(logger, "_is_customized") or not logger._is_customized:
    logger.add(sys.stderr, format=log_format, filter=lambda x: should_log(x) and emoji_filter(x))
    logger._is_customized = True

custom_logger = logger



NLP_TASKS = {
    "text_binary_classification": 1,
    "text_multi_class_classification": 2,
    "text_entity_extraction": 4,
    "text_extractive_question_answering": 5,
    "text_summarization": 8,
    "text_single_column_regression": 10,
    "speech_recognition": 11,
    "natural_language_inference": 22,
    "lm_training": 9,
}

VISION_TASKS = {
    "image_binary_classification": 17,
    "image_multi_class_classification": 18,
    "image_single_column_regression": 24,
    "dreambooth": 25,
}

TABULAR_TASKS = {
    "tabular_binary_classification": 13,
    "tabular_multi_class_classification": 14,
    "tabular_multi_label_classification": 15,
    "tabular_single_column_regression": 16,
    "tabular": 26,
}


TASKS = {
    **NLP_TASKS,
    **VISION_TASKS,
    **TABULAR_TASKS,
}

COLUMN_MAPPING = {
    "text_binary_classification": ("text", "label"),
    "text_multi_class_classification": ("text", "label"),
    "text_entity_extraction": ("text", "tokens"),
    "text_extractive_question_answering": ("text", "context", "question", "answer"),
    "text_summarization": ("text", "summary"),
    "text_single_column_regression": ("text", "label"),
    "speech_recognition": ("audio", "text"),
    "natural_language_inference": ("premise", "hypothesis", "label"),
    "image_binary_classification": ("image", "label"),
    "image_multi_class_classification": ("image", "label"),
    "image_single_column_regression": ("image", "label"),
    # "dreambooth": ("image", "label"),
    "tabular_binary_classification": ("id", "label"),
    "tabular_multi_class_classification": ("id", "label"),
    "tabular_multi_label_classification": ("id", "label"),
    "tabular_single_column_regression": ("id", "label"),
    "lm_training": ("text", "prompt_start", "prompt", "context", "response"),
}

TASK_TYPE_MAPPING = {
    "text_binary_classification": "Natural Language Processing",
    "text_multi_class_classification": "Natural Language Processing",
    "text_entity_extraction": "Natural Language Processing",
    "text_extractive_question_answering": "Natural Language Processing",
    "text_summarization": "Natural Language Processing",
    "text_single_column_regression": "Natural Language Processing",
    "lm_training": "Natural Language Processing",
    "speech_recognition": "Natural Language Processing",
    "natural_language_inference": "Natural Language Processing",
    "image_binary_classification": "Computer Vision",
    "image_multi_class_classification": "Computer Vision",
    "image_single_column_regression": "Computer Vision",
    "dreambooth": "Computer Vision",
    "tabular_binary_classification": "Tabular",
    "tabular_multi_class_classification": "Tabular",
    "tabular_multi_label_classification": "Tabular",
    "tabular_single_column_regression": "Tabular",
}



import json
import os
import random
import string
import zipfile

import gradio as gr
import pandas as pd
from huggingface_hub import list_models

from autotrain import logger
from autotrain.dataset import AutoTrainDataset, AutoTrainDreamboothDataset, AutoTrainImageClassificationDataset
from autotrain.languages import SUPPORTED_LANGUAGES
from autotrain.params import Params
from autotrain.project import Project
from autotrain.utils import get_project_cost, get_user_token, user_authentication


APP_TASKS = {
    "Natural Language Processing": ["Text Classification"],
    # "Tabular": TABULAR_TASKS,
    "Computer Vision": ["Image Classification", "Dreambooth"],
}

APP_TASKS_MAPPING = {
    "Text Classification": "text_multi_class_classification",
    "LLM Finetuning": "lm_training",
    "Image Classification": "image_multi_class_classification",
    "Dreambooth": "dreambooth",
}

APP_TASK_TYPE_MAPPING = {
    "text_classification": "Natural Language Processing",
    "lm_training": "Natural Language Processing",
    "image_classification": "Computer Vision",
    "dreambooth": "Computer Vision",
}

ALLOWED_FILE_TYPES = [
    ".csv",
    ".CSV",
    ".jsonl",
    ".JSONL",
    ".zip",
    ".ZIP",
    ".png",
    ".PNG",
    ".jpg",
    ".JPG",
    ".jpeg",
    ".JPEG",
]


def _login_user(user_token):
    user_info = user_authentication(token=user_token)
    username = user_info["name"]

    user_can_pay = user_info["canPay"]
    orgs = user_info["orgs"]

    valid_orgs = [org for org in orgs if org["canPay"] is True]
    valid_orgs = [org for org in valid_orgs if org["roleInOrg"] in ("admin", "write")]
    valid_orgs = [org["name"] for org in valid_orgs]

    valid_can_pay = [username] + valid_orgs if user_can_pay else valid_orgs
    who_is_training = [username] + [org["name"] for org in orgs]
    return user_token, valid_can_pay, who_is_training


def _update_task_type(project_type):
    return gr.Dropdown.update(
        value=APP_TASKS[project_type][0],
        choices=APP_TASKS[project_type],
        visible=True,
    )


def _update_model_choice(task, autotrain_backend):
    # TODO: add tabular and remember, for tabular, we only support AutoTrain
    if autotrain_backend.lower() != "huggingface internal":
        model_choice = ["HuggingFace Hub"]
        return gr.Dropdown.update(
            value=model_choice[0],
            choices=model_choice,
            visible=True,
        )

    if task == "LLM Finetuning":
        model_choice = ["HuggingFace Hub"]
    else:
        model_choice = ["AutoTrain", "HuggingFace Hub"]

    return gr.Dropdown.update(
        value=model_choice[0],
        choices=model_choice,
        visible=True,
    )


def _update_file_type(task):
    task = APP_TASKS_MAPPING[task]
    if task in ("text_multi_class_classification", "lm_training"):
        return gr.Radio.update(
            value="CSV",
            choices=["CSV", "JSONL"],
            visible=True,
        )
    elif task == "image_multi_class_classification":
        return gr.Radio.update(
            value="ZIP",
            choices=["Image Subfolders", "ZIP"],
            visible=True,
        )
    elif task == "dreambooth":
        return gr.Radio.update(
            value="ZIP",
            choices=["Image Folder", "ZIP"],
            visible=True,
        )
    else:
        raise NotImplementedError


def _update_param_choice(model_choice, autotrain_backend):
    logger.info(f"model_choice: {model_choice}")
    choices = ["AutoTrain", "Manual"] if model_choice == "HuggingFace Hub" else ["AutoTrain"]
    choices = ["Manual"] if autotrain_backend != "HuggingFace Internal" else choices
    return gr.Dropdown.update(
        value=choices[0],
        choices=choices,
        visible=True,
    )


def _project_type_update(project_type, task_type, autotrain_backend):
    logger.info(f"project_type: {project_type}, task_type: {task_type}")
    task_choices_update = _update_task_type(project_type)
    model_choices_update = _update_model_choice(task_choices_update["value"], autotrain_backend)
    param_choices_update = _update_param_choice(model_choices_update["value"], autotrain_backend)
    return [
        task_choices_update,
        model_choices_update,
        param_choices_update,
        _update_hub_model_choices(task_choices_update["value"], model_choices_update["value"]),
    ]


def _task_type_update(task_type, autotrain_backend):
    logger.info(f"task_type: {task_type}")
    model_choices_update = _update_model_choice(task_type, autotrain_backend)
    param_choices_update = _update_param_choice(model_choices_update["value"], autotrain_backend)
    return [
        model_choices_update,
        param_choices_update,
        _update_hub_model_choices(task_type, model_choices_update["value"]),
    ]


def _update_col_map(training_data, task):
    task = APP_TASKS_MAPPING[task]
    if task == "text_multi_class_classification":
        data_cols = pd.read_csv(training_data[0].name, nrows=2).columns.tolist()
        return [
            gr.Dropdown.update(visible=True, choices=data_cols, label="Map `text` column", value=data_cols[0]),
            gr.Dropdown.update(visible=True, choices=data_cols, label="Map `target` column", value=data_cols[1]),
            gr.Text.update(visible=False),
        ]
    elif task == "lm_training":
        data_cols = pd.read_csv(training_data[0].name, nrows=2).columns.tolist()
        return [
            gr.Dropdown.update(visible=True, choices=data_cols, label="Map `text` column", value=data_cols[0]),
            gr.Dropdown.update(visible=False),
            gr.Text.update(visible=False),
        ]
    elif task == "dreambooth":
        return [
            gr.Dropdown.update(visible=False),
            gr.Dropdown.update(visible=False),
            gr.Text.update(visible=True, label="Concept Token", interactive=True),
        ]
    else:
        return [
            gr.Dropdown.update(visible=False),
            gr.Dropdown.update(visible=False),
            gr.Text.update(visible=False),
        ]


def _estimate_costs(
    training_data, validation_data, task, user_token, autotrain_username, training_params_txt, autotrain_backend
):
    if autotrain_backend.lower() != "huggingface internal":
        return [
            gr.Markdown.update(
                value="Cost estimation is not available for this backend",
                visible=True,
            ),
            gr.Number.update(visible=False),
        ]
    try:
        logger.info("Estimating costs....")
        if training_data is None:
            return [
                gr.Markdown.update(
                    value="Could not estimate cost. Please add training data",
                    visible=True,
                ),
                gr.Number.update(visible=False),
            ]
        if validation_data is None:
            validation_data = []

        training_params = json.loads(training_params_txt)
        if len(training_params) == 0:
            return [
                gr.Markdown.update(
                    value="Could not estimate cost. Please add atleast one job",
                    visible=True,
                ),
                gr.Number.update(visible=False),
            ]
        elif len(training_params) == 1:
            if "num_models" in training_params[0]:
                num_models = training_params[0]["num_models"]
            else:
                num_models = 1
        else:
            num_models = len(training_params)
        task = APP_TASKS_MAPPING[task]
        num_samples = 0
        logger.info("Estimating number of samples")
        if task in ("text_multi_class_classification", "lm_training"):
            for _f in training_data:
                num_samples += pd.read_csv(_f.name).shape[0]
            for _f in validation_data:
                num_samples += pd.read_csv(_f.name).shape[0]
        elif task == "image_multi_class_classification":
            logger.info(f"training_data: {training_data}")
            if len(training_data) > 1:
                return [
                    gr.Markdown.update(
                        value="Only one training file is supported for image classification",
                        visible=True,
                    ),
                    gr.Number.update(visible=False),
                ]
            if len(validation_data) > 1:
                return [
                    gr.Markdown.update(
                        value="Only one validation file is supported for image classification",
                        visible=True,
                    ),
                    gr.Number.update(visible=False),
                ]
            for _f in training_data:
                zip_ref = zipfile.ZipFile(_f.name, "r")
                for _ in zip_ref.namelist():
                    num_samples += 1
            for _f in validation_data:
                zip_ref = zipfile.ZipFile(_f.name, "r")
                for _ in zip_ref.namelist():
                    num_samples += 1
        elif task == "dreambooth":
            num_samples = len(training_data)
        else:
            raise NotImplementedError

        logger.info(f"Estimating costs for: num_models: {num_models}, task: {task}, num_samples: {num_samples}")
        estimated_cost = get_project_cost(
            username=autotrain_username,
            token=user_token,
            task=task,
            num_samples=num_samples,
            num_models=num_models,
        )
        logger.info(f"Estimated_cost: {estimated_cost}")
        return [
            gr.Markdown.update(
                value=f"Estimated cost: ${estimated_cost:.2f}. Note: clicking on 'Create Project' will start training and incur charges!",
                visible=True,
            ),
            gr.Number.update(visible=False),
        ]
    except Exception as e:
        logger.error(e)
        logger.error("Could not estimate cost, check inputs")
        return [
            gr.Markdown.update(
                value="Could not estimate cost, check inputs",
                visible=True,
            ),
            gr.Number.update(visible=False),
        ]


def get_job_params(param_choice, training_params, task):
    if param_choice == "autotrain":
        if len(training_params) > 1:
            raise ValueError("❌ Only one job parameter is allowed for AutoTrain.")
        training_params[0].update({"task": task})
    elif param_choice.lower() == "manual":
        for i in range(len(training_params)):
            training_params[i].update({"task": task})
            if "hub_model" in training_params[i]:
                # remove hub_model from training_params
                training_params[i].pop("hub_model")
    return training_params


def _update_project_name():
    random_project_name = "-".join(
        ["".join(random.choices(string.ascii_lowercase + string.digits, k=4)) for _ in range(3)]
    )
    # check if training tracker exists
    if os.path.exists(os.path.join("/tmp", "training")):
        return [
            gr.Text.update(value=random_project_name, visible=True, interactive=True),
            gr.Button.update(interactive=False),
        ]
    return [
        gr.Text.update(value=random_project_name, visible=True, interactive=True),
        gr.Button.update(interactive=True),
    ]


def _update_hub_model_choices(task, model_choice):
    task = APP_TASKS_MAPPING[task]
    logger.info(f"Updating hub model choices for task: {task}, model_choice: {model_choice}")
    if model_choice.lower() == "autotrain":
        return gr.Dropdown.update(
            visible=False,
            interactive=False,
        )
    if task == "text_multi_class_classification":
        hub_models1 = list_models(filter="fill-mask", sort="downloads", direction=-1, limit=100)
        hub_models2 = list_models(filter="text-classification", sort="downloads", direction=-1, limit=100)
        hub_models = list(hub_models1) + list(hub_models2)
    elif task == "lm_training":
        hub_models = list(list_models(filter="text-generation", sort="downloads", direction=-1, limit=100))
    elif task == "image_multi_class_classification":
        hub_models = list(list_models(filter="image-classification", sort="downloads", direction=-1, limit=100))
    elif task == "dreambooth":
        hub_models = list(list_models(filter="text-to-image", sort="downloads", direction=-1, limit=100))
    else:
        raise NotImplementedError
    # sort by number of downloads in descending order
    hub_models = [{"id": m.modelId, "downloads": m.downloads} for m in hub_models if m.private is False]
    hub_models = sorted(hub_models, key=lambda x: x["downloads"], reverse=True)

    if task == "dreambooth":
        choices = ["stabilityai/stable-diffusion-xl-base-1.0"] + [m["id"] for m in hub_models]
        value = choices[0]
        return gr.Dropdown.update(
            choices=choices,
            value=value,
            visible=True,
            interactive=True,
        )

    return gr.Dropdown.update(
        choices=[m["id"] for m in hub_models],
        value=hub_models[0]["id"],
        visible=True,
        interactive=True,
    )


def _update_backend(backend):
    if backend != "Hugging Face Internal":
        return [
            gr.Dropdown.update(
                visible=True,
                interactive=True,
                choices=["HuggingFace Hub"],
                value="HuggingFace Hub",
            ),
            gr.Dropdown.update(
                visible=True,
                interactive=True,
                choices=["Manual"],
                value="Manual",
            ),
        ]
    return [
        gr.Dropdown.update(
            visible=True,
            interactive=True,
        ),
        gr.Dropdown.update(
            visible=True,
            interactive=True,
        ),
    ]


def _create_project(
    autotrain_username,
    valid_can_pay,
    project_name,
    user_token,
    task,
    training_data,
    validation_data,
    col_map_text,
    col_map_label,
    concept_token,
    training_params_txt,
    hub_model,
    estimated_cost,
    autotrain_backend,
):
    task = APP_TASKS_MAPPING[task]
    valid_can_pay = valid_can_pay.split(",")
    can_pay = autotrain_username in valid_can_pay
    logger.info(f"🚨🚨🚨Creating project: {project_name}")
    logger.info(f"🚨Task: {task}")
    logger.info(f"🚨Training data: {training_data}")
    logger.info(f"🚨Validation data: {validation_data}")
    logger.info(f"🚨Training params: {training_params_txt}")
    logger.info(f"🚨Hub model: {hub_model}")
    logger.info(f"🚨Estimated cost: {estimated_cost}")
    logger.info(f"🚨:Can pay: {can_pay}")

    if can_pay is False and estimated_cost > 0:
        raise gr.Error("❌ You do not have enough credits to create this project. Please add a valid payment method.")

    training_params = json.loads(training_params_txt)
    if len(training_params) == 0:
        raise gr.Error("Please add atleast one job")
    elif len(training_params) == 1:
        if "num_models" in training_params[0]:
            param_choice = "autotrain"
        else:
            param_choice = "manual"
    else:
        param_choice = "manual"

    if task == "image_multi_class_classification":
        training_data = training_data[0].name
        if validation_data is not None:
            validation_data = validation_data[0].name
        dset = AutoTrainImageClassificationDataset(
            train_data=training_data,
            token=user_token,
            project_name=project_name,
            username=autotrain_username,
            valid_data=validation_data,
            percent_valid=None,  # TODO: add to UI
        )
    elif task == "text_multi_class_classification":
        training_data = [f.name for f in training_data]
        if validation_data is None:
            validation_data = []
        else:
            validation_data = [f.name for f in validation_data]
        dset = AutoTrainDataset(
            train_data=training_data,
            task=task,
            token=user_token,
            project_name=project_name,
            username=autotrain_username,
            column_mapping={"text": col_map_text, "label": col_map_label},
            valid_data=validation_data,
            percent_valid=None,  # TODO: add to UI
        )
    elif task == "lm_training":
        training_data = [f.name for f in training_data]
        if validation_data is None:
            validation_data = []
        else:
            validation_data = [f.name for f in validation_data]
        dset = AutoTrainDataset(
            train_data=training_data,
            task=task,
            token=user_token,
            project_name=project_name,
            username=autotrain_username,
            column_mapping={"text": col_map_text},
            valid_data=validation_data,
            percent_valid=None,  # TODO: add to UI
        )
    elif task == "dreambooth":
        dset = AutoTrainDreamboothDataset(
            concept_images=training_data,
            concept_name=concept_token,
            token=user_token,
            project_name=project_name,
            username=autotrain_username,
        )
    else:
        raise NotImplementedError

    dset.prepare()
    project = Project(
        dataset=dset,
        param_choice=param_choice,
        hub_model=hub_model,
        job_params=get_job_params(param_choice, training_params, task),
    )
    if autotrain_backend.lower() == "huggingface internal":
        project_id = project.create()
        project.approve(project_id)
        return gr.Markdown.update(
            value=f"Project created successfully. Monitor progess on the [dashboard](https://ui.autotrain.huggingface.co/{project_id}/trainings).",
            visible=True,
        )
    else:
        project.create(local=True)


def get_variable_name(var, namespace):
    for name in namespace:
        if namespace[name] is var:
            return name
    return None


def disable_create_project_button():
    return gr.Button.update(interactive=False)


def main():
    with gr.Blocks(theme="freddyaboulton/dracula_revamped") as demo:
        gr.Markdown("## 🤗 AutoTrain Advanced")
        user_token = os.environ.get("HF_TOKEN", "")

        if len(user_token) == 0:
            user_token = get_user_token()

        if user_token is None:
            gr.Markdown(
                """Please login with a write [token](https://huggingface.co/settings/tokens).
                Pass your HF token in an environment variable called `HF_TOKEN` and then restart this app.
                """
            )
            return demo

        user_token, valid_can_pay, who_is_training = _login_user(user_token)

        if user_token is None or len(user_token) == 0:
            gr.Error("Please login with a write token.")

        user_token = gr.Textbox(
            value=user_token, type="password", lines=1, max_lines=1, visible=False, interactive=False
        )
        valid_can_pay = gr.Textbox(value=",".join(valid_can_pay), visible=False, interactive=False)
        with gr.Row():
            with gr.Column():
                with gr.Row():
                    autotrain_username = gr.Dropdown(
                        label="AutoTrain Username",
                        choices=who_is_training,
                        value=who_is_training[0] if who_is_training else "",
                    )
                    autotrain_backend = gr.Dropdown(
                        label="AutoTrain Backend",
                        choices=["HuggingFace Internal", "HuggingFace Spaces"],
                        value="HuggingFace Internal",
                        interactive=True,
                    )
                with gr.Row():
                    project_name = gr.Textbox(label="Project name", value="", lines=1, max_lines=1, interactive=True)
                    project_type = gr.Dropdown(
                        label="Project Type", choices=list(APP_TASKS.keys()), value=list(APP_TASKS.keys())[0]
                    )
                    task_type = gr.Dropdown(
                        label="Task",
                        choices=APP_TASKS[list(APP_TASKS.keys())[0]],
                        value=APP_TASKS[list(APP_TASKS.keys())[0]][0],
                        interactive=True,
                    )
                    model_choice = gr.Dropdown(
                        label="Model Choice",
                        choices=["AutoTrain", "HuggingFace Hub"],
                        value="AutoTrain",
                        visible=True,
                        interactive=True,
                    )
                hub_model = gr.Dropdown(
                    label="Hub Model",
                    value="",
                    visible=False,
                    interactive=True,
                    elem_id="hub_model",
                )
        gr.Markdown("<hr>")
        with gr.Row():
            with gr.Column():
                with gr.Tabs(elem_id="tabs"):
                    with gr.TabItem("Data"):
                        with gr.Column():
                            # file_type_training = gr.Radio(
                            #     label="File Type",
                            #     choices=["CSV", "JSONL"],
                            #     value="CSV",
                            #     visible=True,
                            #     interactive=True,
                            # )
                            training_data = gr.File(
                                label="Training Data",
                                file_types=ALLOWED_FILE_TYPES,
                                file_count="multiple",
                                visible=True,
                                interactive=True,
                                elem_id="training_data_box",
                            )
                            with gr.Accordion("Validation Data (Optional)", open=False):
                                validation_data = gr.File(
                                    label="Validation Data (Optional)",
                                    file_types=ALLOWED_FILE_TYPES,
                                    file_count="multiple",
                                    visible=True,
                                    interactive=True,
                                    elem_id="validation_data_box",
                                )
                            with gr.Row():
                                col_map_text = gr.Dropdown(
                                    label="Text Column", choices=[], visible=False, interactive=True
                                )
                                col_map_target = gr.Dropdown(
                                    label="Target Column", choices=[], visible=False, interactive=True
                                )
                                concept_token = gr.Text(
                                    value="", visible=False, interactive=True, lines=1, max_lines=1
                                )
                    with gr.TabItem("Params"):
                        with gr.Row():
                            source_language = gr.Dropdown(
                                label="Source Language",
                                choices=SUPPORTED_LANGUAGES[:-1],
                                value="en",
                                visible=True,
                                interactive=True,
                                elem_id="source_language",
                            )
                            num_models = gr.Slider(
                                label="Number of Models",
                                minimum=1,
                                maximum=25,
                                value=5,
                                step=1,
                                visible=True,
                                interactive=True,
                                elem_id="num_models",
                            )
                            target_language = gr.Dropdown(
                                label="Target Language",
                                choices=["fr"],
                                value="fr",
                                visible=False,
                                interactive=True,
                                elem_id="target_language",
                            )
                            image_size = gr.Number(
                                label="Image Size",
                                value=512,
                                visible=False,
                                interactive=True,
                                elem_id="image_size",
                            )

                        with gr.Row():
                            learning_rate = gr.Number(
                                label="Learning Rate",
                                value=5e-5,
                                visible=False,
                                interactive=True,
                                elem_id="learning_rate",
                            )
                            batch_size = gr.Number(
                                label="Train Batch Size",
                                value=32,
                                visible=False,
                                interactive=True,
                                elem_id="train_batch_size",
                            )
                            num_epochs = gr.Number(
                                label="Number of Epochs",
                                value=3,
                                visible=False,
                                interactive=True,
                                elem_id="num_train_epochs",
                            )
                        with gr.Row():
                            gradient_accumulation_steps = gr.Number(
                                label="Gradient Accumulation Steps",
                                value=1,
                                visible=False,
                                interactive=True,
                                elem_id="gradient_accumulation_steps",
                            )
                            percentage_warmup_steps = gr.Number(
                                label="Percentage of Warmup Steps",
                                value=0.1,
                                visible=False,
                                interactive=True,
                                elem_id="percentage_warmup",
                            )
                            weight_decay = gr.Number(
                                label="Weight Decay",
                                value=0.01,
                                visible=False,
                                interactive=True,
                                elem_id="weight_decay",
                            )
                        with gr.Row():
                            lora_r = gr.Number(
                                label="LoraR",
                                value=16,
                                visible=False,
                                interactive=True,
                                elem_id="lora_r",
                            )
                            lora_alpha = gr.Number(
                                label="LoraAlpha",
                                value=32,
                                visible=False,
                                interactive=True,
                                elem_id="lora_alpha",
                            )
                            lora_dropout = gr.Number(
                                label="Lora Dropout",
                                value=0.1,
                                visible=False,
                                interactive=True,
                                elem_id="lora_dropout",
                            )
                        with gr.Row():
                            db_num_steps = gr.Number(
                                label="Num Steps",
                                value=500,
                                visible=False,
                                interactive=True,
                                elem_id="num_steps",
                            )
                        with gr.Row():
                            optimizer = gr.Dropdown(
                                label="Optimizer",
                                choices=["adamw_torch", "adamw_hf", "sgd", "adafactor", "adagrad"],
                                value="adamw_torch",
                                visible=False,
                                interactive=True,
                                elem_id="optimizer",
                            )
                            scheduler = gr.Dropdown(
                                label="Scheduler",
                                choices=["linear", "cosine"],
                                value="linear",
                                visible=False,
                                interactive=True,
                                elem_id="scheduler",
                            )

                        add_job_button = gr.Button(
                            value="Add Job",
                            visible=True,
                            interactive=True,
                            elem_id="add_job",
                        )
                        # clear_jobs_button = gr.Button(
                        #     value="Clear Jobs",
                        #     visible=True,
                        #     interactive=True,
                        #     elem_id="clear_jobs",
                        # )
                gr.Markdown("<hr>")
                estimated_costs_md = gr.Markdown(value="Estimated Costs: N/A", visible=True, interactive=False)
                estimated_costs_num = gr.Number(value=0, visible=False, interactive=False)
                create_project_button = gr.Button(
                    value="Create Project",
                    visible=True,
                    interactive=True,
                    elem_id="create_project",
                )
            with gr.Column():
                param_choice = gr.Dropdown(
                    label="Param Choice",
                    choices=["AutoTrain"],
                    value="AutoTrain",
                    visible=True,
                    interactive=True,
                )
                training_params_txt = gr.Text(value="[]", visible=False, interactive=False)
                training_params_md = gr.DataFrame(visible=False, interactive=False)

        final_output = gr.Markdown(value="", visible=True, interactive=False)
        hyperparameters = [
            hub_model,
            num_models,
            source_language,
            target_language,
            learning_rate,
            batch_size,
            num_epochs,
            gradient_accumulation_steps,
            lora_r,
            lora_alpha,
            lora_dropout,
            optimizer,
            scheduler,
            percentage_warmup_steps,
            weight_decay,
            db_num_steps,
            image_size,
        ]

        def _update_params(params_data):
            _task = params_data[task_type]
            _task = APP_TASKS_MAPPING[_task]
            params = Params(
                task=_task,
                param_choice="autotrain" if params_data[param_choice] == "AutoTrain" else "manual",
                model_choice="autotrain" if params_data[model_choice] == "AutoTrain" else "hub_model",
            )
            params = params.get()
            visible_params = []
            for param in hyperparameters:
                if param.elem_id in params.keys():
                    visible_params.append(param.elem_id)
            op = [h.update(visible=h.elem_id in visible_params) for h in hyperparameters]
            op.append(add_job_button.update(visible=True))
            op.append(training_params_md.update(visible=False))
            op.append(training_params_txt.update(value="[]"))
            return op

        autotrain_backend.change(
            _project_type_update,
            inputs=[project_type, task_type, autotrain_backend],
            outputs=[task_type, model_choice, param_choice, hub_model],
        )

        project_type.change(
            _project_type_update,
            inputs=[project_type, task_type, autotrain_backend],
            outputs=[task_type, model_choice, param_choice, hub_model],
        )
        task_type.change(
            _task_type_update,
            inputs=[task_type, autotrain_backend],
            outputs=[model_choice, param_choice, hub_model],
        )
        model_choice.change(
            _update_param_choice,
            inputs=[model_choice, autotrain_backend],
            outputs=param_choice,
        ).then(
            _update_hub_model_choices,
            inputs=[task_type, model_choice],
            outputs=hub_model,
        )

        param_choice.change(
            _update_params,
            inputs=set([task_type, param_choice, model_choice] + hyperparameters + [add_job_button]),
            outputs=hyperparameters + [add_job_button, training_params_md, training_params_txt],
        )
        task_type.change(
            _update_params,
            inputs=set([task_type, param_choice, model_choice] + hyperparameters + [add_job_button]),
            outputs=hyperparameters + [add_job_button, training_params_md, training_params_txt],
        )
        model_choice.change(
            _update_params,
            inputs=set([task_type, param_choice, model_choice] + hyperparameters + [add_job_button]),
            outputs=hyperparameters + [add_job_button, training_params_md, training_params_txt],
        )

        def _add_job(params_data):
            _task = params_data[task_type]
            _task = APP_TASKS_MAPPING[_task]
            _param_choice = "autotrain" if params_data[param_choice] == "AutoTrain" else "manual"
            _model_choice = "autotrain" if params_data[model_choice] == "AutoTrain" else "hub_model"
            if _model_choice == "hub_model" and params_data[hub_model] is None:
                logger.error("Hub model is None")
                return
            _training_params = {}
            params = Params(task=_task, param_choice=_param_choice, model_choice=_model_choice)
            params = params.get()
            for _param in hyperparameters:
                if _param.elem_id in params.keys():
                    _training_params[_param.elem_id] = params_data[_param]
            _training_params_md = json.loads(params_data[training_params_txt])
            if _param_choice == "autotrain":
                if len(_training_params_md) > 0:
                    _training_params_md[0] = _training_params
                    _training_params_md = _training_params_md[:1]
                else:
                    _training_params_md.append(_training_params)
            else:
                _training_params_md.append(_training_params)
            params_df = pd.DataFrame(_training_params_md)
            # remove hub_model column
            if "hub_model" in params_df.columns:
                params_df = params_df.drop(columns=["hub_model"])
            return [
                gr.DataFrame.update(value=params_df, visible=True),
                gr.Textbox.update(value=json.dumps(_training_params_md), visible=False),
            ]

        add_job_button.click(
            _add_job,
            inputs=set(
                [task_type, param_choice, model_choice] + hyperparameters + [training_params_md, training_params_txt]
            ),
            outputs=[training_params_md, training_params_txt],
        )
        col_map_components = [
            col_map_text,
            col_map_target,
            concept_token,
        ]
        training_data.change(
            _update_col_map,
            inputs=[training_data, task_type],
            outputs=col_map_components,
        )
        task_type.change(
            _update_col_map,
            inputs=[training_data, task_type],
            outputs=col_map_components,
        )
        estimate_costs_inputs = [
            training_data,
            validation_data,
            task_type,
            user_token,
            autotrain_username,
            training_params_txt,
            autotrain_backend,
        ]
        estimate_costs_outputs = [estimated_costs_md, estimated_costs_num]
        training_data.change(_estimate_costs, inputs=estimate_costs_inputs, outputs=estimate_costs_outputs)
        validation_data.change(_estimate_costs, inputs=estimate_costs_inputs, outputs=estimate_costs_outputs)
        training_params_txt.change(_estimate_costs, inputs=estimate_costs_inputs, outputs=estimate_costs_outputs)
        task_type.change(_estimate_costs, inputs=estimate_costs_inputs, outputs=estimate_costs_outputs)
        add_job_button.click(_estimate_costs, inputs=estimate_costs_inputs, outputs=estimate_costs_outputs)

        create_project_button.click(disable_create_project_button, None, create_project_button).then(
            _create_project,
            inputs=[
                autotrain_username,
                valid_can_pay,
                project_name,
                user_token,
                task_type,
                training_data,
                validation_data,
                col_map_text,
                col_map_target,
                concept_token,
                training_params_txt,
                hub_model,
                estimated_costs_num,
                autotrain_backend,
            ],
            outputs=final_output,
        )

        demo.load(
            _update_project_name,
            outputs=[project_name, create_project_button],
        )

    return demo
